"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class TargetingEndAge(str, Enum):
    SEVENTEEN = "Seventeen"
    TWENTY = "Twenty"
    TWENTY_FOUR = "TwentyFour"
    TWENTY_NINE = "TwentyNine"
    THIRTY_FOUR = "ThirtyFour"
    THIRTY_NINE = "ThirtyNine"
    FORTY_FOUR = "FortyFour"
    FORTY_NINE = "FortyNine"
    FIFTY_FOUR = "FiftyFour"
    SIXTY_FOUR = "SixtyFour"
    SIXTY_FOUR_PLUS = "SixtyFourPlus"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .workflowcallbackinput import WorkflowCallbackInput, WorkflowCallbackInputTypedDict
import pydantic
from pydantic import model_serializer
from ttd_workflows.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ThirdPartyDataInputTypedDict(TypedDict):
    r"""Required fields for submitting a bulk job for third-party data."""

    partner_id: str
    r"""The partner ID to query for."""
    query_shape: NotRequired[Nullable[str]]
    r"""The shape of the query with the fields being asked for, which is sent downstream.
    This determines what the response will look like.

    For example, a query shape equal to `\"nodes {id name}\"` will return the `id` and `name` fields only.

    If this is not provided the default query shape will be used:

    ```graphql
    nodes {
    id
    name
    providerId
    providerElementId
    description
    buyable
    fullPath
    activeUniques {
    householdCount
    idsConnectedTvCount
    idsInAppCount
    idsWebCount
    personsCount
    thirdPartyDataOverlapCount
    lastUpdated
    }
    }
    ```
    """
    callback_input: NotRequired[WorkflowCallbackInputTypedDict]


class ThirdPartyDataInput(BaseModel):
    r"""Required fields for submitting a bulk job for third-party data."""

    partner_id: Annotated[str, pydantic.Field(alias="partnerId")]
    r"""The partner ID to query for."""

    query_shape: Annotated[
        OptionalNullable[str], pydantic.Field(alias="queryShape")
    ] = UNSET
    r"""The shape of the query with the fields being asked for, which is sent downstream.
    This determines what the response will look like.

    For example, a query shape equal to `\"nodes {id name}\"` will return the `id` and `name` fields only.

    If this is not provided the default query shape will be used:

    ```graphql
    nodes {
    id
    name
    providerId
    providerElementId
    description
    buyable
    fullPath
    activeUniques {
    householdCount
    idsConnectedTvCount
    idsInAppCount
    idsWebCount
    personsCount
    thirdPartyDataOverlapCount
    lastUpdated
    }
    }
    ```
    """

    callback_input: Annotated[
        Optional[WorkflowCallbackInput], pydantic.Field(alias="callbackInput")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["queryShape", "callbackInput"]
        nullable_fields = ["queryShape"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
import pydantic
from ttd_workflows.types import BaseModel
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SubmitRestRequestResponseBodyTypedDict(TypedDict):
    r"""OK"""


class SubmitRestRequestResponseBody(BaseModel):
    r"""OK"""


class SubmitRestRequestResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    object: NotRequired[SubmitRestRequestResponseBodyTypedDict]
    r"""OK"""


class SubmitRestRequestResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    object: Optional[SubmitRestRequestResponseBody] = None
    r"""OK"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .adgroupupdateworkflowinput import (
    AdGroupUpdateWorkflowInput,
    AdGroupUpdateWorkflowInputTypedDict,
)
from .workflowcallbackinput import WorkflowCallbackInput, WorkflowCallbackInputTypedDict
import pydantic
from pydantic import model_serializer
from ttd_workflows.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AdGroupBulkUpdateWorkflowInputWithValidationTypedDict(TypedDict):
    input: Nullable[List[AdGroupUpdateWorkflowInputTypedDict]]
    validate_input_only: NotRequired[Nullable[bool]]
    callback_input: NotRequired[WorkflowCallbackInputTypedDict]


class AdGroupBulkUpdateWorkflowInputWithValidation(BaseModel):
    input: Nullable[List[AdGroupUpdateWorkflowInput]]

    validate_input_only: Annotated[
        OptionalNullable[bool], pydantic.Field(alias="validateInputOnly")
    ] = UNSET

    callback_input: Annotated[
        Optional[WorkflowCallbackInput], pydantic.Field(alias="callbackInput")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["validateInputOnly", "callbackInput"]
        nullable_fields = ["input", "validateInputOnly"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customroasconfig import CustomROASConfig, CustomROASConfigTypedDict
import pydantic
from pydantic import model_serializer
from ttd_workflows.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CampaignWorkflowCampaignConversionReportingColumnInputTypedDict(TypedDict):
    tracking_tag_id: str
    include_in_custom_cpa: bool
    reporting_column_id: int
    roas_config: NotRequired[CustomROASConfigTypedDict]
    weight: NotRequired[Nullable[float]]
    cross_device_attribution_model_id: NotRequired[Nullable[str]]


class CampaignWorkflowCampaignConversionReportingColumnInput(BaseModel):
    tracking_tag_id: Annotated[str, pydantic.Field(alias="trackingTagId")]

    include_in_custom_cpa: Annotated[bool, pydantic.Field(alias="includeInCustomCPA")]

    reporting_column_id: Annotated[int, pydantic.Field(alias="reportingColumnId")]

    roas_config: Annotated[
        Optional[CustomROASConfig], pydantic.Field(alias="ROASConfig")
    ] = None

    weight: OptionalNullable[float] = UNSET

    cross_device_attribution_model_id: Annotated[
        OptionalNullable[str], pydantic.Field(alias="crossDeviceAttributionModelId")
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["ROASConfig", "weight", "crossDeviceAttributionModelId"]
        nullable_fields = ["weight", "crossDeviceAttributionModelId"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .adgrouppayload import AdGroupPayload, AdGroupPayloadTypedDict
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
import pydantic
from ttd_workflows.types import BaseModel
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateAdGroupResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    ad_group_payload: NotRequired[AdGroupPayloadTypedDict]
    r"""Created"""


class CreateAdGroupResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    ad_group_payload: Optional[AdGroupPayload] = None
    r"""Created"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
from .standardjobsubmitresponse import (
    StandardJobSubmitResponse,
    StandardJobSubmitResponseTypedDict,
)
import pydantic
from ttd_workflows.types import BaseModel
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateCampaignsJobResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    standard_job_submit_response: NotRequired[StandardJobSubmitResponseTypedDict]
    r"""Accepted"""


class CreateCampaignsJobResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    standard_job_submit_response: Optional[StandardJobSubmitResponse] = None
    r"""Accepted"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class RestAPIMethodType(str, Enum):
    POST = "POST"
    PUT = "PUT"
    GET = "GET"
    DELETE = "DELETE"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
import pydantic
from ttd_workflows.types import BaseModel
from typing import Any, Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SubmitGraphQlRequestResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    object: NotRequired[Dict[str, Any]]
    r"""OK"""


class SubmitGraphQlRequestResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    object: Optional[Dict[str, Any]] = None
    r"""OK"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
import pydantic
from ttd_workflows.types import BaseModel
from ttd_workflows.utils import FieldMetadata, QueryParamMetadata, RequestMetadata
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ArchiveCampaignsRequestTypedDict(TypedDict):
    force_archive: NotRequired[bool]
    request_body: NotRequired[List[str]]


class ArchiveCampaignsRequest(BaseModel):
    force_archive: Annotated[
        Optional[bool],
        pydantic.Field(alias="forceArchive"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = False

    request_body: Annotated[
        Optional[List[str]],
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ] = None


class ArchiveCampaignsResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    strings: NotRequired[List[str]]
    r"""OK"""


class ArchiveCampaignsResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    strings: Optional[List[str]] = None
    r"""OK"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .problemdetails import ProblemDetails
import httpx
import pydantic
from ttd_workflows.models import WorkflowsError
from ttd_workflows.types import BaseModel, OptionalNullable, UNSET
from typing import Dict, Optional
from typing_extensions import Annotated


class ProblemDetailsErrorData(BaseModel):
    type: OptionalNullable[str] = UNSET

    title: OptionalNullable[str] = UNSET

    status: OptionalNullable[int] = UNSET

    detail: OptionalNullable[str] = UNSET

    instance: OptionalNullable[str] = UNSET

    additional_properties: Annotated[
        Optional[Dict[str, ProblemDetails]], pydantic.Field(exclude=True)
    ] = None


class ProblemDetailsError(WorkflowsError):
    data: ProblemDetailsErrorData

    def __init__(
        self,
        data: ProblemDetailsErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        self.data = data



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class CampaignPacingMode(str, Enum):
    PACE_AHEAD = "PaceAhead"
    PACE_EVENLY = "PaceEvenly"
    PACE_TO_DAILY_CAP = "PaceToDailyCap"
    PACE_AS_SOON_AS_POSSIBLE = "PaceAsSoonAsPossible"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .graphqlqueryjobresponse import (
    GraphQlQueryJobResponse,
    GraphQlQueryJobResponseTypedDict,
)
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
import pydantic
from ttd_workflows.types import BaseModel
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SubmitGraphQlQueryJobResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    graph_ql_query_job_response: NotRequired[GraphQlQueryJobResponseTypedDict]
    r"""OK"""


class SubmitGraphQlQueryJobResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    graph_ql_query_job_response: Optional[GraphQlQueryJobResponse] = None
    r"""OK"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class AllocationType(str, Enum):
    MINIMUM = "Minimum"
    FIXED = "Fixed"
    MAXIMUM = "Maximum"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from ttd_workflows.types import BaseModel
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AdGroupKoaOptimizationSettingsTypedDict(TypedDict):
    are_future_koa_features_enabled: NotRequired[bool]
    predictive_clearing_enabled: NotRequired[bool]


class AdGroupKoaOptimizationSettings(BaseModel):
    are_future_koa_features_enabled: Annotated[
        Optional[bool], pydantic.Field(alias="areFutureKoaFeaturesEnabled")
    ] = None

    predictive_clearing_enabled: Annotated[
        Optional[bool], pydantic.Field(alias="predictiveClearingEnabled")
    ] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .workflowstatus import WorkflowStatus
from datetime import datetime
import pydantic
from pydantic import model_serializer
from ttd_workflows.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing_extensions import Annotated, NotRequired, TypedDict


class StandardJobStatusResponseTypedDict(TypedDict):
    id: int
    r"""The ID of the job."""
    created_at_utc: datetime
    r"""The UTC date and time that the job was created."""
    status: WorkflowStatus
    completed_at_utc: NotRequired[Nullable[datetime]]
    r"""The UTC data and time that the job completed."""
    completion_percentage: NotRequired[Nullable[float]]
    r"""The job completion percentage."""
    url: NotRequired[Nullable[str]]
    r"""The URL from which the job result can be downloaded."""
    raw_result: NotRequired[Nullable[str]]
    r"""The raw job result if the response is less than or equal to 20MB in size.
    If the job result is larger than 20MB, then null is returned.
    """
    errors: NotRequired[Nullable[str]]
    r"""A list of errors encountered during workflow processing."""


class StandardJobStatusResponse(BaseModel):
    id: int
    r"""The ID of the job."""

    created_at_utc: Annotated[datetime, pydantic.Field(alias="createdAtUtc")]
    r"""The UTC date and time that the job was created."""

    status: WorkflowStatus

    completed_at_utc: Annotated[
        OptionalNullable[datetime], pydantic.Field(alias="completedAtUtc")
    ] = UNSET
    r"""The UTC data and time that the job completed."""

    completion_percentage: Annotated[
        OptionalNullable[float], pydantic.Field(alias="completionPercentage")
    ] = UNSET
    r"""The job completion percentage."""

    url: OptionalNullable[str] = UNSET
    r"""The URL from which the job result can be downloaded."""

    raw_result: Annotated[OptionalNullable[str], pydantic.Field(alias="rawResult")] = (
        UNSET
    )
    r"""The raw job result if the response is less than or equal to 20MB in size.
    If the job result is larger than 20MB, then null is returned.
    """

    errors: OptionalNullable[str] = UNSET
    r"""A list of errors encountered during workflow processing."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "completedAtUtc",
            "completionPercentage",
            "url",
            "rawResult",
            "errors",
        ]
        nullable_fields = [
            "completedAtUtc",
            "completionPercentage",
            "url",
            "rawResult",
            "errors",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class AdGroupFunnelLocation(str, Enum):
    NONE = "None"
    AWARENESS = "Awareness"
    CONSIDERATION = "Consideration"
    CONVERSION = "Conversion"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from typing import TYPE_CHECKING
from importlib import import_module

if TYPE_CHECKING:
    from .adgroupassociatebidlist import (
        AdGroupAssociateBidList,
        AdGroupAssociateBidListTypedDict,
    )
    from .adgroupaudiencetargeting import (
        AdGroupAudienceTargeting,
        AdGroupAudienceTargetingTypedDict,
    )
    from .adgroupbudget import AdGroupBudget, AdGroupBudgetTypedDict
    from .adgroupbulkcreateworkflowinputwithvalidation import (
        AdGroupBulkCreateWorkflowInputWithValidation,
        AdGroupBulkCreateWorkflowInputWithValidationTypedDict,
    )
    from .adgroupbulkupdateworkflowinputwithvalidation import (
        AdGroupBulkUpdateWorkflowInputWithValidation,
        AdGroupBulkUpdateWorkflowInputWithValidationTypedDict,
    )
    from .adgroupchannel import AdGroupChannel
    from .adgroupcomscoresettings import (
        AdGroupComscoreSettings,
        AdGroupComscoreSettingsTypedDict,
    )
    from .adgroupcontracttargeting import (
        AdGroupContractTargeting,
        AdGroupContractTargetingTypedDict,
    )
    from .adgroupcreateworkflowinput import (
        AdGroupCreateWorkflowInput,
        AdGroupCreateWorkflowInputTypedDict,
    )
    from .adgroupcreateworkflowinputwithvalidation import (
        AdGroupCreateWorkflowInputWithValidation,
        AdGroupCreateWorkflowInputWithValidationTypedDict,
    )
    from .adgroupcreateworkflowprimaryinput import (
        AdGroupCreateWorkflowPrimaryInput,
        AdGroupCreateWorkflowPrimaryInputTypedDict,
    )
    from .adgroupflight import AdGroupFlight, AdGroupFlightTypedDict
    from .adgroupfunnellocation import AdGroupFunnelLocation
    from .adgroupkoaoptimizationsettings import (
        AdGroupKoaOptimizationSettings,
        AdGroupKoaOptimizationSettingsTypedDict,
    )
    from .adgroupnielsentrackingattributes import (
        AdGroupNielsenTrackingAttributes,
        AdGroupNielsenTrackingAttributesTypedDict,
    )
    from .adgrouppayload import AdGroupPayload, AdGroupPayloadTypedDict
    from .adgrouproigoal import AdGroupROIGoal, AdGroupROIGoalTypedDict
    from .adgroupupdateworkflowinput import (
        AdGroupUpdateWorkflowInput,
        AdGroupUpdateWorkflowInputTypedDict,
    )
    from .adgroupupdateworkflowinputwithvalidation import (
        AdGroupUpdateWorkflowInputWithValidation,
        AdGroupUpdateWorkflowInputWithValidationTypedDict,
    )
    from .adgroupupdateworkflowprimaryinput import (
        AdGroupUpdateWorkflowPrimaryInput,
        AdGroupUpdateWorkflowPrimaryInputTypedDict,
    )
    from .adgroupworkflow import AdGroupWorkflow, AdGroupWorkflowTypedDict
    from .adgroupworkflowadvancedinput import (
        AdGroupWorkflowAdvancedInput,
        AdGroupWorkflowAdvancedInputTypedDict,
    )
    from .adgroupworkflowassociatebidlistinput import (
        AdGroupWorkflowAssociateBidListInput,
        AdGroupWorkflowAssociateBidListInputTypedDict,
    )
    from .adgroupworkflowaudiencetargetinginput import (
        AdGroupWorkflowAudienceTargetingInput,
        AdGroupWorkflowAudienceTargetingInputTypedDict,
    )
    from .adgroupworkflowbudgetinput import (
        AdGroupWorkflowBudgetInput,
        AdGroupWorkflowBudgetInputTypedDict,
    )
    from .adgroupworkflowcomscoresettingsinput import (
        AdGroupWorkflowComscoreSettingsInput,
        AdGroupWorkflowComscoreSettingsInputTypedDict,
    )
    from .adgroupworkflowcontracttargetinginput import (
        AdGroupWorkflowContractTargetingInput,
        AdGroupWorkflowContractTargetingInputTypedDict,
    )
    from .adgroupworkflowflightinput import (
        AdGroupWorkflowFlightInput,
        AdGroupWorkflowFlightInputTypedDict,
    )
    from .adgroupworkflowkoaoptimizationsettingsinput import (
        AdGroupWorkflowKoaOptimizationSettingsInput,
        AdGroupWorkflowKoaOptimizationSettingsInputTypedDict,
    )
    from .adgroupworkflownewfrequencyconfiginput import (
        AdGroupWorkflowNewFrequencyConfigInput,
        AdGroupWorkflowNewFrequencyConfigInputTypedDict,
    )
    from .adgroupworkflownielsentrackingattributesinput import (
        AdGroupWorkflowNielsenTrackingAttributesInput,
        AdGroupWorkflowNielsenTrackingAttributesInputTypedDict,
    )
    from .adgroupworkflowroigoalinput import (
        AdGroupWorkflowROIGoalInput,
        AdGroupWorkflowROIGoalInputTypedDict,
    )
    from .allocationtype import AllocationType
    from .apierror import APIError
    from .archiveadgroupsop import (
        ArchiveAdGroupsRequest,
        ArchiveAdGroupsRequestTypedDict,
        ArchiveAdGroupsResponse,
        ArchiveAdGroupsResponseTypedDict,
    )
    from .archivecampaignsop import (
        ArchiveCampaignsRequest,
        ArchiveCampaignsRequestTypedDict,
        ArchiveCampaignsResponse,
        ArchiveCampaignsResponseTypedDict,
    )
    from .bulkjobstatus import BulkJobStatus
    from .callrestapiworkflowinput import (
        CallRestAPIWorkflowInput,
        CallRestAPIWorkflowInputTypedDict,
    )
    from .campaignbulkcreateworkflowinputwithvalidation import (
        CampaignBulkCreateWorkflowInputWithValidation,
        CampaignBulkCreateWorkflowInputWithValidationTypedDict,
    )
    from .campaignbulkupdateworkflowinputwithvalidation import (
        CampaignBulkUpdateWorkflowInputWithValidation,
        CampaignBulkUpdateWorkflowInputWithValidationTypedDict,
    )
    from .campaignchanneltype import CampaignChannelType
    from .campaigncreateworkflowadgroupadvancedinput import (
        CampaignCreateWorkflowAdGroupAdvancedInput,
        CampaignCreateWorkflowAdGroupAdvancedInputTypedDict,
    )
    from .campaigncreateworkflowadgroupflightinput import (
        CampaignCreateWorkflowAdGroupFlightInput,
        CampaignCreateWorkflowAdGroupFlightInputTypedDict,
    )
    from .campaigncreateworkflowadgroupinput import (
        CampaignCreateWorkflowAdGroupInput,
        CampaignCreateWorkflowAdGroupInputTypedDict,
    )
    from .campaigncreateworkflowinput import (
        CampaignCreateWorkflowInput,
        CampaignCreateWorkflowInputTypedDict,
    )
    from .campaigncreateworkflowinputwithvalidation import (
        CampaignCreateWorkflowInputWithValidation,
        CampaignCreateWorkflowInputWithValidationTypedDict,
    )
    from .campaigncreateworkflowprimaryinput import (
        CampaignCreateWorkflowPrimaryInput,
        CampaignCreateWorkflowPrimaryInputTypedDict,
    )
    from .campaignflightworkflow import (
        CampaignFlightWorkflow,
        CampaignFlightWorkflowTypedDict,
    )
    from .campaignpacingmode import CampaignPacingMode
    from .campaignpayload import CampaignPayload, CampaignPayloadTypedDict
    from .campaignupdateworkflowinput import (
        CampaignUpdateWorkflowInput,
        CampaignUpdateWorkflowInputTypedDict,
    )
    from .campaignupdateworkflowinputwithvalidation import (
        CampaignUpdateWorkflowInputWithValidation,
        CampaignUpdateWorkflowInputWithValidationTypedDict,
    )
    from .campaignupdateworkflowprimaryinput import (
        CampaignUpdateWorkflowPrimaryInput,
        CampaignUpdateWorkflowPrimaryInputTypedDict,
    )
    from .campaignversionworkflow import (
        CampaignVersionWorkflow,
        CampaignVersionWorkflowTypedDict,
    )
    from .campaignworkflow import CampaignWorkflow, CampaignWorkflowTypedDict
    from .campaignworkflowadvancedinput import (
        CampaignWorkflowAdvancedInput,
        CampaignWorkflowAdvancedInputTypedDict,
    )
    from .campaignworkflowbudgetinput import (
        CampaignWorkflowBudgetInput,
        CampaignWorkflowBudgetInputTypedDict,
    )
    from .campaignworkflowcampaignconversionreportingcolumninput import (
        CampaignWorkflowCampaignConversionReportingColumnInput,
        CampaignWorkflowCampaignConversionReportingColumnInputTypedDict,
    )
    from .campaignworkflowflightinput import (
        CampaignWorkflowFlightInput,
        CampaignWorkflowFlightInputTypedDict,
    )
    from .campaignworkflowroigoalinput import (
        CampaignWorkflowROIGoalInput,
        CampaignWorkflowROIGoalInputTypedDict,
    )
    from .createadgroupop import CreateAdGroupResponse, CreateAdGroupResponseTypedDict
    from .createadgroupsjobop import (
        CreateAdGroupsJobResponse,
        CreateAdGroupsJobResponseTypedDict,
    )
    from .createcampaignop import (
        CreateCampaignResponse,
        CreateCampaignResponseTypedDict,
    )
    from .createcampaignsjobop import (
        CreateCampaignsJobResponse,
        CreateCampaignsJobResponseTypedDict,
    )
    from .customcpatype import CustomCPAType
    from .customroasconfig import CustomROASConfig, CustomROASConfigTypedDict
    from .dimensionalbiddingdimensions import DimensionalBiddingDimensions
    from .enhancednielsenreportingoptions import EnhancedNielsenReportingOptions
    from .firstpartydatainput import FirstPartyDataInput, FirstPartyDataInputTypedDict
    from .getcampaignversionop import (
        GetCampaignVersionRequest,
        GetCampaignVersionRequestTypedDict,
        GetCampaignVersionResponse,
        GetCampaignVersionResponseTypedDict,
    )
    from .getfirstpartydatajobop import (
        GetFirstPartyDataJobResponse,
        GetFirstPartyDataJobResponseTypedDict,
    )
    from .getgraphqlqueryjobstatusop import (
        GetGraphQlQueryJobStatusRequest,
        GetGraphQlQueryJobStatusRequestTypedDict,
        GetGraphQlQueryJobStatusResponse,
        GetGraphQlQueryJobStatusResponseTypedDict,
    )
    from .getjobstatusop import (
        GetJobStatusRequest,
        GetJobStatusRequestTypedDict,
        GetJobStatusResponse,
        GetJobStatusResponseTypedDict,
    )
    from .getthirdpartydatajobop import (
        GetThirdPartyDataJobResponse,
        GetThirdPartyDataJobResponseTypedDict,
    )
    from .graphqljobcallbackinput import (
        GraphQlJobCallbackInput,
        GraphQlJobCallbackInputTypedDict,
    )
    from .graphqlqueryjob import GraphQlQueryJob, GraphQlQueryJobTypedDict
    from .graphqlqueryjoberror import (
        GraphQLQueryJobError,
        GraphQLQueryJobErrorTypedDict,
    )
    from .graphqlqueryjobinput import (
        GraphQlQueryJobInput,
        GraphQlQueryJobInputTypedDict,
    )
    from .graphqlqueryjobpayload import (
        GraphQlQueryJobPayload,
        GraphQlQueryJobPayloadTypedDict,
    )
    from .graphqlqueryjobresponse import (
        GraphQlQueryJobResponse,
        GraphQlQueryJobResponseTypedDict,
    )
    from .graphqlqueryjobretrievalresponse import (
        GraphQLQueryJobRetrievalResponse,
        GraphQLQueryJobRetrievalResponseTypedDict,
    )
    from .graphqlrequestinput import (
        GraphQLRequestInput,
        GraphQLRequestInputTypedDict,
        Variables,
        VariablesTypedDict,
    )
    from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
    from .keyvaluepairofstringandstringinput import (
        KeyValuePairOfStringAndStringInput,
        KeyValuePairOfStringAndStringInputTypedDict,
    )
    from .no_response_error import NoResponseError
    from .problemdetails import ProblemDetails, ProblemDetailsTypedDict
    from .problemdetails_error import ProblemDetailsError, ProblemDetailsErrorData
    from .responsevalidationerror import ResponseValidationError
    from .restapimethodtype import RestAPIMethodType
    from .security import Security, SecurityTypedDict
    from .standardjobstatusresponse import (
        StandardJobStatusResponse,
        StandardJobStatusResponseTypedDict,
    )
    from .standardjobsubmitresponse import (
        StandardJobSubmitResponse,
        StandardJobSubmitResponseTypedDict,
    )
    from .submitgraphqlqueryjobop import (
        SubmitGraphQlQueryJobResponse,
        SubmitGraphQlQueryJobResponseTypedDict,
    )
    from .submitgraphqlrequestop import (
        SubmitGraphQlRequestResponse,
        SubmitGraphQlRequestResponseTypedDict,
    )
    from .submitrestrequestop import (
        SubmitRestRequestResponse,
        SubmitRestRequestResponseBody,
        SubmitRestRequestResponseBodyTypedDict,
        SubmitRestRequestResponseTypedDict,
    )
    from .targetingendage import TargetingEndAge
    from .targetinggender import TargetingGender
    from .targetingstartage import TargetingStartAge
    from .thirdpartydatainput import ThirdPartyDataInput, ThirdPartyDataInputTypedDict
    from .updateadgroupop import UpdateAdGroupResponse, UpdateAdGroupResponseTypedDict
    from .updateadgroupsjobop import (
        UpdateAdGroupsJobResponse,
        UpdateAdGroupsJobResponseTypedDict,
    )
    from .updatecampaignop import (
        UpdateCampaignResponse,
        UpdateCampaignResponseTypedDict,
    )
    from .updatecampaignsjobop import (
        UpdateCampaignsJobResponse,
        UpdateCampaignsJobResponseTypedDict,
    )
    from .workflowcallbackinput import (
        WorkflowCallbackInput,
        WorkflowCallbackInputTypedDict,
    )
    from .workflowserror import WorkflowsError
    from .workflowstatus import WorkflowStatus

__all__ = [
    "APIError",
    "AdGroupAssociateBidList",
    "AdGroupAssociateBidListTypedDict",
    "AdGroupAudienceTargeting",
    "AdGroupAudienceTargetingTypedDict",
    "AdGroupBudget",
    "AdGroupBudgetTypedDict",
    "AdGroupBulkCreateWorkflowInputWithValidation",
    "AdGroupBulkCreateWorkflowInputWithValidationTypedDict",
    "AdGroupBulkUpdateWorkflowInputWithValidation",
    "AdGroupBulkUpdateWorkflowInputWithValidationTypedDict",
    "AdGroupChannel",
    "AdGroupComscoreSettings",
    "AdGroupComscoreSettingsTypedDict",
    "AdGroupContractTargeting",
    "AdGroupContractTargetingTypedDict",
    "AdGroupCreateWorkflowInput",
    "AdGroupCreateWorkflowInputTypedDict",
    "AdGroupCreateWorkflowInputWithValidation",
    "AdGroupCreateWorkflowInputWithValidationTypedDict",
    "AdGroupCreateWorkflowPrimaryInput",
    "AdGroupCreateWorkflowPrimaryInputTypedDict",
    "AdGroupFlight",
    "AdGroupFlightTypedDict",
    "AdGroupFunnelLocation",
    "AdGroupKoaOptimizationSettings",
    "AdGroupKoaOptimizationSettingsTypedDict",
    "AdGroupNielsenTrackingAttributes",
    "AdGroupNielsenTrackingAttributesTypedDict",
    "AdGroupPayload",
    "AdGroupPayloadTypedDict",
    "AdGroupROIGoal",
    "AdGroupROIGoalTypedDict",
    "AdGroupUpdateWorkflowInput",
    "AdGroupUpdateWorkflowInputTypedDict",
    "AdGroupUpdateWorkflowInputWithValidation",
    "AdGroupUpdateWorkflowInputWithValidationTypedDict",
    "AdGroupUpdateWorkflowPrimaryInput",
    "AdGroupUpdateWorkflowPrimaryInputTypedDict",
    "AdGroupWorkflow",
    "AdGroupWorkflowAdvancedInput",
    "AdGroupWorkflowAdvancedInputTypedDict",
    "AdGroupWorkflowAssociateBidListInput",
    "AdGroupWorkflowAssociateBidListInputTypedDict",
    "AdGroupWorkflowAudienceTargetingInput",
    "AdGroupWorkflowAudienceTargetingInputTypedDict",
    "AdGroupWorkflowBudgetInput",
    "AdGroupWorkflowBudgetInputTypedDict",
    "AdGroupWorkflowComscoreSettingsInput",
    "AdGroupWorkflowComscoreSettingsInputTypedDict",
    "AdGroupWorkflowContractTargetingInput",
    "AdGroupWorkflowContractTargetingInputTypedDict",
    "AdGroupWorkflowFlightInput",
    "AdGroupWorkflowFlightInputTypedDict",
    "AdGroupWorkflowKoaOptimizationSettingsInput",
    "AdGroupWorkflowKoaOptimizationSettingsInputTypedDict",
    "AdGroupWorkflowNewFrequencyConfigInput",
    "AdGroupWorkflowNewFrequencyConfigInputTypedDict",
    "AdGroupWorkflowNielsenTrackingAttributesInput",
    "AdGroupWorkflowNielsenTrackingAttributesInputTypedDict",
    "AdGroupWorkflowROIGoalInput",
    "AdGroupWorkflowROIGoalInputTypedDict",
    "AdGroupWorkflowTypedDict",
    "AllocationType",
    "ArchiveAdGroupsRequest",
    "ArchiveAdGroupsRequestTypedDict",
    "ArchiveAdGroupsResponse",
    "ArchiveAdGroupsResponseTypedDict",
    "ArchiveCampaignsRequest",
    "ArchiveCampaignsRequestTypedDict",
    "ArchiveCampaignsResponse",
    "ArchiveCampaignsResponseTypedDict",
    "BulkJobStatus",
    "CallRestAPIWorkflowInput",
    "CallRestAPIWorkflowInputTypedDict",
    "CampaignBulkCreateWorkflowInputWithValidation",
    "CampaignBulkCreateWorkflowInputWithValidationTypedDict",
    "CampaignBulkUpdateWorkflowInputWithValidation",
    "CampaignBulkUpdateWorkflowInputWithValidationTypedDict",
    "CampaignChannelType",
    "CampaignCreateWorkflowAdGroupAdvancedInput",
    "CampaignCreateWorkflowAdGroupAdvancedInputTypedDict",
    "CampaignCreateWorkflowAdGroupFlightInput",
    "CampaignCreateWorkflowAdGroupFlightInputTypedDict",
    "CampaignCreateWorkflowAdGroupInput",
    "CampaignCreateWorkflowAdGroupInputTypedDict",
    "CampaignCreateWorkflowInput",
    "CampaignCreateWorkflowInputTypedDict",
    "CampaignCreateWorkflowInputWithValidation",
    "CampaignCreateWorkflowInputWithValidationTypedDict",
    "CampaignCreateWorkflowPrimaryInput",
    "CampaignCreateWorkflowPrimaryInputTypedDict",
    "CampaignFlightWorkflow",
    "CampaignFlightWorkflowTypedDict",
    "CampaignPacingMode",
    "CampaignPayload",
    "CampaignPayloadTypedDict",
    "CampaignUpdateWorkflowInput",
    "CampaignUpdateWorkflowInputTypedDict",
    "CampaignUpdateWorkflowInputWithValidation",
    "CampaignUpdateWorkflowInputWithValidationTypedDict",
    "CampaignUpdateWorkflowPrimaryInput",
    "CampaignUpdateWorkflowPrimaryInputTypedDict",
    "CampaignVersionWorkflow",
    "CampaignVersionWorkflowTypedDict",
    "CampaignWorkflow",
    "CampaignWorkflowAdvancedInput",
    "CampaignWorkflowAdvancedInputTypedDict",
    "CampaignWorkflowBudgetInput",
    "CampaignWorkflowBudgetInputTypedDict",
    "CampaignWorkflowCampaignConversionReportingColumnInput",
    "CampaignWorkflowCampaignConversionReportingColumnInputTypedDict",
    "CampaignWorkflowFlightInput",
    "CampaignWorkflowFlightInputTypedDict",
    "CampaignWorkflowROIGoalInput",
    "CampaignWorkflowROIGoalInputTypedDict",
    "CampaignWorkflowTypedDict",
    "CreateAdGroupResponse",
    "CreateAdGroupResponseTypedDict",
    "CreateAdGroupsJobResponse",
    "CreateAdGroupsJobResponseTypedDict",
    "CreateCampaignResponse",
    "CreateCampaignResponseTypedDict",
    "CreateCampaignsJobResponse",
    "CreateCampaignsJobResponseTypedDict",
    "CustomCPAType",
    "CustomROASConfig",
    "CustomROASConfigTypedDict",
    "DimensionalBiddingDimensions",
    "EnhancedNielsenReportingOptions",
    "FirstPartyDataInput",
    "FirstPartyDataInputTypedDict",
    "GetCampaignVersionRequest",
    "GetCampaignVersionRequestTypedDict",
    "GetCampaignVersionResponse",
    "GetCampaignVersionResponseTypedDict",
    "GetFirstPartyDataJobResponse",
    "GetFirstPartyDataJobResponseTypedDict",
    "GetGraphQlQueryJobStatusRequest",
    "GetGraphQlQueryJobStatusRequestTypedDict",
    "GetGraphQlQueryJobStatusResponse",
    "GetGraphQlQueryJobStatusResponseTypedDict",
    "GetJobStatusRequest",
    "GetJobStatusRequestTypedDict",
    "GetJobStatusResponse",
    "GetJobStatusResponseTypedDict",
    "GetThirdPartyDataJobResponse",
    "GetThirdPartyDataJobResponseTypedDict",
    "GraphQLQueryJobError",
    "GraphQLQueryJobErrorTypedDict",
    "GraphQLQueryJobRetrievalResponse",
    "GraphQLQueryJobRetrievalResponseTypedDict",
    "GraphQLRequestInput",
    "GraphQLRequestInputTypedDict",
    "GraphQlJobCallbackInput",
    "GraphQlJobCallbackInputTypedDict",
    "GraphQlQueryJob",
    "GraphQlQueryJobInput",
    "GraphQlQueryJobInputTypedDict",
    "GraphQlQueryJobPayload",
    "GraphQlQueryJobPayloadTypedDict",
    "GraphQlQueryJobResponse",
    "GraphQlQueryJobResponseTypedDict",
    "GraphQlQueryJobTypedDict",
    "HTTPMetadata",
    "HTTPMetadataTypedDict",
    "KeyValuePairOfStringAndStringInput",
    "KeyValuePairOfStringAndStringInputTypedDict",
    "NoResponseError",
    "ProblemDetails",
    "ProblemDetailsError",
    "ProblemDetailsErrorData",
    "ProblemDetailsTypedDict",
    "ResponseValidationError",
    "RestAPIMethodType",
    "Security",
    "SecurityTypedDict",
    "StandardJobStatusResponse",
    "StandardJobStatusResponseTypedDict",
    "StandardJobSubmitResponse",
    "StandardJobSubmitResponseTypedDict",
    "SubmitGraphQlQueryJobResponse",
    "SubmitGraphQlQueryJobResponseTypedDict",
    "SubmitGraphQlRequestResponse",
    "SubmitGraphQlRequestResponseTypedDict",
    "SubmitRestRequestResponse",
    "SubmitRestRequestResponseBody",
    "SubmitRestRequestResponseBodyTypedDict",
    "SubmitRestRequestResponseTypedDict",
    "TargetingEndAge",
    "TargetingGender",
    "TargetingStartAge",
    "ThirdPartyDataInput",
    "ThirdPartyDataInputTypedDict",
    "UpdateAdGroupResponse",
    "UpdateAdGroupResponseTypedDict",
    "UpdateAdGroupsJobResponse",
    "UpdateAdGroupsJobResponseTypedDict",
    "UpdateCampaignResponse",
    "UpdateCampaignResponseTypedDict",
    "UpdateCampaignsJobResponse",
    "UpdateCampaignsJobResponseTypedDict",
    "Variables",
    "VariablesTypedDict",
    "WorkflowCallbackInput",
    "WorkflowCallbackInputTypedDict",
    "WorkflowStatus",
    "WorkflowsError",
]

_dynamic_imports: dict[str, str] = {
    "AdGroupAssociateBidList": ".adgroupassociatebidlist",
    "AdGroupAssociateBidListTypedDict": ".adgroupassociatebidlist",
    "AdGroupAudienceTargeting": ".adgroupaudiencetargeting",
    "AdGroupAudienceTargetingTypedDict": ".adgroupaudiencetargeting",
    "AdGroupBudget": ".adgroupbudget",
    "AdGroupBudgetTypedDict": ".adgroupbudget",
    "AdGroupBulkCreateWorkflowInputWithValidation": ".adgroupbulkcreateworkflowinputwithvalidation",
    "AdGroupBulkCreateWorkflowInputWithValidationTypedDict": ".adgroupbulkcreateworkflowinputwithvalidation",
    "AdGroupBulkUpdateWorkflowInputWithValidation": ".adgroupbulkupdateworkflowinputwithvalidation",
    "AdGroupBulkUpdateWorkflowInputWithValidationTypedDict": ".adgroupbulkupdateworkflowinputwithvalidation",
    "AdGroupChannel": ".adgroupchannel",
    "AdGroupComscoreSettings": ".adgroupcomscoresettings",
    "AdGroupComscoreSettingsTypedDict": ".adgroupcomscoresettings",
    "AdGroupContractTargeting": ".adgroupcontracttargeting",
    "AdGroupContractTargetingTypedDict": ".adgroupcontracttargeting",
    "AdGroupCreateWorkflowInput": ".adgroupcreateworkflowinput",
    "AdGroupCreateWorkflowInputTypedDict": ".adgroupcreateworkflowinput",
    "AdGroupCreateWorkflowInputWithValidation": ".adgroupcreateworkflowinputwithvalidation",
    "AdGroupCreateWorkflowInputWithValidationTypedDict": ".adgroupcreateworkflowinputwithvalidation",
    "AdGroupCreateWorkflowPrimaryInput": ".adgroupcreateworkflowprimaryinput",
    "AdGroupCreateWorkflowPrimaryInputTypedDict": ".adgroupcreateworkflowprimaryinput",
    "AdGroupFlight": ".adgroupflight",
    "AdGroupFlightTypedDict": ".adgroupflight",
    "AdGroupFunnelLocation": ".adgroupfunnellocation",
    "AdGroupKoaOptimizationSettings": ".adgroupkoaoptimizationsettings",
    "AdGroupKoaOptimizationSettingsTypedDict": ".adgroupkoaoptimizationsettings",
    "AdGroupNielsenTrackingAttributes": ".adgroupnielsentrackingattributes",
    "AdGroupNielsenTrackingAttributesTypedDict": ".adgroupnielsentrackingattributes",
    "AdGroupPayload": ".adgrouppayload",
    "AdGroupPayloadTypedDict": ".adgrouppayload",
    "AdGroupROIGoal": ".adgrouproigoal",
    "AdGroupROIGoalTypedDict": ".adgrouproigoal",
    "AdGroupUpdateWorkflowInput": ".adgroupupdateworkflowinput",
    "AdGroupUpdateWorkflowInputTypedDict": ".adgroupupdateworkflowinput",
    "AdGroupUpdateWorkflowInputWithValidation": ".adgroupupdateworkflowinputwithvalidation",
    "AdGroupUpdateWorkflowInputWithValidationTypedDict": ".adgroupupdateworkflowinputwithvalidation",
    "AdGroupUpdateWorkflowPrimaryInput": ".adgroupupdateworkflowprimaryinput",
    "AdGroupUpdateWorkflowPrimaryInputTypedDict": ".adgroupupdateworkflowprimaryinput",
    "AdGroupWorkflow": ".adgroupworkflow",
    "AdGroupWorkflowTypedDict": ".adgroupworkflow",
    "AdGroupWorkflowAdvancedInput": ".adgroupworkflowadvancedinput",
    "AdGroupWorkflowAdvancedInputTypedDict": ".adgroupworkflowadvancedinput",
    "AdGroupWorkflowAssociateBidListInput": ".adgroupworkflowassociatebidlistinput",
    "AdGroupWorkflowAssociateBidListInputTypedDict": ".adgroupworkflowassociatebidlistinput",
    "AdGroupWorkflowAudienceTargetingInput": ".adgroupworkflowaudiencetargetinginput",
    "AdGroupWorkflowAudienceTargetingInputTypedDict": ".adgroupworkflowaudiencetargetinginput",
    "AdGroupWorkflowBudgetInput": ".adgroupworkflowbudgetinput",
    "AdGroupWorkflowBudgetInputTypedDict": ".adgroupworkflowbudgetinput",
    "AdGroupWorkflowComscoreSettingsInput": ".adgroupworkflowcomscoresettingsinput",
    "AdGroupWorkflowComscoreSettingsInputTypedDict": ".adgroupworkflowcomscoresettingsinput",
    "AdGroupWorkflowContractTargetingInput": ".adgroupworkflowcontracttargetinginput",
    "AdGroupWorkflowContractTargetingInputTypedDict": ".adgroupworkflowcontracttargetinginput",
    "AdGroupWorkflowFlightInput": ".adgroupworkflowflightinput",
    "AdGroupWorkflowFlightInputTypedDict": ".adgroupworkflowflightinput",
    "AdGroupWorkflowKoaOptimizationSettingsInput": ".adgroupworkflowkoaoptimizationsettingsinput",
    "AdGroupWorkflowKoaOptimizationSettingsInputTypedDict": ".adgroupworkflowkoaoptimizationsettingsinput",
    "AdGroupWorkflowNewFrequencyConfigInput": ".adgroupworkflownewfrequencyconfiginput",
    "AdGroupWorkflowNewFrequencyConfigInputTypedDict": ".adgroupworkflownewfrequencyconfiginput",
    "AdGroupWorkflowNielsenTrackingAttributesInput": ".adgroupworkflownielsentrackingattributesinput",
    "AdGroupWorkflowNielsenTrackingAttributesInputTypedDict": ".adgroupworkflownielsentrackingattributesinput",
    "AdGroupWorkflowROIGoalInput": ".adgroupworkflowroigoalinput",
    "AdGroupWorkflowROIGoalInputTypedDict": ".adgroupworkflowroigoalinput",
    "AllocationType": ".allocationtype",
    "APIError": ".apierror",
    "ArchiveAdGroupsRequest": ".archiveadgroupsop",
    "ArchiveAdGroupsRequestTypedDict": ".archiveadgroupsop",
    "ArchiveAdGroupsResponse": ".archiveadgroupsop",
    "ArchiveAdGroupsResponseTypedDict": ".archiveadgroupsop",
    "ArchiveCampaignsRequest": ".archivecampaignsop",
    "ArchiveCampaignsRequestTypedDict": ".archivecampaignsop",
    "ArchiveCampaignsResponse": ".archivecampaignsop",
    "ArchiveCampaignsResponseTypedDict": ".archivecampaignsop",
    "BulkJobStatus": ".bulkjobstatus",
    "CallRestAPIWorkflowInput": ".callrestapiworkflowinput",
    "CallRestAPIWorkflowInputTypedDict": ".callrestapiworkflowinput",
    "CampaignBulkCreateWorkflowInputWithValidation": ".campaignbulkcreateworkflowinputwithvalidation",
    "CampaignBulkCreateWorkflowInputWithValidationTypedDict": ".campaignbulkcreateworkflowinputwithvalidation",
    "CampaignBulkUpdateWorkflowInputWithValidation": ".campaignbulkupdateworkflowinputwithvalidation",
    "CampaignBulkUpdateWorkflowInputWithValidationTypedDict": ".campaignbulkupdateworkflowinputwithvalidation",
    "CampaignChannelType": ".campaignchanneltype",
    "CampaignCreateWorkflowAdGroupAdvancedInput": ".campaigncreateworkflowadgroupadvancedinput",
    "CampaignCreateWorkflowAdGroupAdvancedInputTypedDict": ".campaigncreateworkflowadgroupadvancedinput",
    "CampaignCreateWorkflowAdGroupFlightInput": ".campaigncreateworkflowadgroupflightinput",
    "CampaignCreateWorkflowAdGroupFlightInputTypedDict": ".campaigncreateworkflowadgroupflightinput",
    "CampaignCreateWorkflowAdGroupInput": ".campaigncreateworkflowadgroupinput",
    "CampaignCreateWorkflowAdGroupInputTypedDict": ".campaigncreateworkflowadgroupinput",
    "CampaignCreateWorkflowInput": ".campaigncreateworkflowinput",
    "CampaignCreateWorkflowInputTypedDict": ".campaigncreateworkflowinput",
    "CampaignCreateWorkflowInputWithValidation": ".campaigncreateworkflowinputwithvalidation",
    "CampaignCreateWorkflowInputWithValidationTypedDict": ".campaigncreateworkflowinputwithvalidation",
    "CampaignCreateWorkflowPrimaryInput": ".campaigncreateworkflowprimaryinput",
    "CampaignCreateWorkflowPrimaryInputTypedDict": ".campaigncreateworkflowprimaryinput",
    "CampaignFlightWorkflow": ".campaignflightworkflow",
    "CampaignFlightWorkflowTypedDict": ".campaignflightworkflow",
    "CampaignPacingMode": ".campaignpacingmode",
    "CampaignPayload": ".campaignpayload",
    "CampaignPayloadTypedDict": ".campaignpayload",
    "CampaignUpdateWorkflowInput": ".campaignupdateworkflowinput",
    "CampaignUpdateWorkflowInputTypedDict": ".campaignupdateworkflowinput",
    "CampaignUpdateWorkflowInputWithValidation": ".campaignupdateworkflowinputwithvalidation",
    "CampaignUpdateWorkflowInputWithValidationTypedDict": ".campaignupdateworkflowinputwithvalidation",
    "CampaignUpdateWorkflowPrimaryInput": ".campaignupdateworkflowprimaryinput",
    "CampaignUpdateWorkflowPrimaryInputTypedDict": ".campaignupdateworkflowprimaryinput",
    "CampaignVersionWorkflow": ".campaignversionworkflow",
    "CampaignVersionWorkflowTypedDict": ".campaignversionworkflow",
    "CampaignWorkflow": ".campaignworkflow",
    "CampaignWorkflowTypedDict": ".campaignworkflow",
    "CampaignWorkflowAdvancedInput": ".campaignworkflowadvancedinput",
    "CampaignWorkflowAdvancedInputTypedDict": ".campaignworkflowadvancedinput",
    "CampaignWorkflowBudgetInput": ".campaignworkflowbudgetinput",
    "CampaignWorkflowBudgetInputTypedDict": ".campaignworkflowbudgetinput",
    "CampaignWorkflowCampaignConversionReportingColumnInput": ".campaignworkflowcampaignconversionreportingcolumninput",
    "CampaignWorkflowCampaignConversionReportingColumnInputTypedDict": ".campaignworkflowcampaignconversionreportingcolumninput",
    "CampaignWorkflowFlightInput": ".campaignworkflowflightinput",
    "CampaignWorkflowFlightInputTypedDict": ".campaignworkflowflightinput",
    "CampaignWorkflowROIGoalInput": ".campaignworkflowroigoalinput",
    "CampaignWorkflowROIGoalInputTypedDict": ".campaignworkflowroigoalinput",
    "CreateAdGroupResponse": ".createadgroupop",
    "CreateAdGroupResponseTypedDict": ".createadgroupop",
    "CreateAdGroupsJobResponse": ".createadgroupsjobop",
    "CreateAdGroupsJobResponseTypedDict": ".createadgroupsjobop",
    "CreateCampaignResponse": ".createcampaignop",
    "CreateCampaignResponseTypedDict": ".createcampaignop",
    "CreateCampaignsJobResponse": ".createcampaignsjobop",
    "CreateCampaignsJobResponseTypedDict": ".createcampaignsjobop",
    "CustomCPAType": ".customcpatype",
    "CustomROASConfig": ".customroasconfig",
    "CustomROASConfigTypedDict": ".customroasconfig",
    "DimensionalBiddingDimensions": ".dimensionalbiddingdimensions",
    "EnhancedNielsenReportingOptions": ".enhancednielsenreportingoptions",
    "FirstPartyDataInput": ".firstpartydatainput",
    "FirstPartyDataInputTypedDict": ".firstpartydatainput",
    "GetCampaignVersionRequest": ".getcampaignversionop",
    "GetCampaignVersionRequestTypedDict": ".getcampaignversionop",
    "GetCampaignVersionResponse": ".getcampaignversionop",
    "GetCampaignVersionResponseTypedDict": ".getcampaignversionop",
    "GetFirstPartyDataJobResponse": ".getfirstpartydatajobop",
    "GetFirstPartyDataJobResponseTypedDict": ".getfirstpartydatajobop",
    "GetGraphQlQueryJobStatusRequest": ".getgraphqlqueryjobstatusop",
    "GetGraphQlQueryJobStatusRequestTypedDict": ".getgraphqlqueryjobstatusop",
    "GetGraphQlQueryJobStatusResponse": ".getgraphqlqueryjobstatusop",
    "GetGraphQlQueryJobStatusResponseTypedDict": ".getgraphqlqueryjobstatusop",
    "GetJobStatusRequest": ".getjobstatusop",
    "GetJobStatusRequestTypedDict": ".getjobstatusop",
    "GetJobStatusResponse": ".getjobstatusop",
    "GetJobStatusResponseTypedDict": ".getjobstatusop",
    "GetThirdPartyDataJobResponse": ".getthirdpartydatajobop",
    "GetThirdPartyDataJobResponseTypedDict": ".getthirdpartydatajobop",
    "GraphQlJobCallbackInput": ".graphqljobcallbackinput",
    "GraphQlJobCallbackInputTypedDict": ".graphqljobcallbackinput",
    "GraphQlQueryJob": ".graphqlqueryjob",
    "GraphQlQueryJobTypedDict": ".graphqlqueryjob",
    "GraphQLQueryJobError": ".graphqlqueryjoberror",
    "GraphQLQueryJobErrorTypedDict": ".graphqlqueryjoberror",
    "GraphQlQueryJobInput": ".graphqlqueryjobinput",
    "GraphQlQueryJobInputTypedDict": ".graphqlqueryjobinput",
    "GraphQlQueryJobPayload": ".graphqlqueryjobpayload",
    "GraphQlQueryJobPayloadTypedDict": ".graphqlqueryjobpayload",
    "GraphQlQueryJobResponse": ".graphqlqueryjobresponse",
    "GraphQlQueryJobResponseTypedDict": ".graphqlqueryjobresponse",
    "GraphQLQueryJobRetrievalResponse": ".graphqlqueryjobretrievalresponse",
    "GraphQLQueryJobRetrievalResponseTypedDict": ".graphqlqueryjobretrievalresponse",
    "GraphQLRequestInput": ".graphqlrequestinput",
    "GraphQLRequestInputTypedDict": ".graphqlrequestinput",
    "Variables": ".graphqlrequestinput",
    "VariablesTypedDict": ".graphqlrequestinput",
    "HTTPMetadata": ".httpmetadata",
    "HTTPMetadataTypedDict": ".httpmetadata",
    "KeyValuePairOfStringAndStringInput": ".keyvaluepairofstringandstringinput",
    "KeyValuePairOfStringAndStringInputTypedDict": ".keyvaluepairofstringandstringinput",
    "NoResponseError": ".no_response_error",
    "ProblemDetails": ".problemdetails",
    "ProblemDetailsTypedDict": ".problemdetails",
    "ProblemDetailsError": ".problemdetails_error",
    "ProblemDetailsErrorData": ".problemdetails_error",
    "ResponseValidationError": ".responsevalidationerror",
    "RestAPIMethodType": ".restapimethodtype",
    "Security": ".security",
    "SecurityTypedDict": ".security",
    "StandardJobStatusResponse": ".standardjobstatusresponse",
    "StandardJobStatusResponseTypedDict": ".standardjobstatusresponse",
    "StandardJobSubmitResponse": ".standardjobsubmitresponse",
    "StandardJobSubmitResponseTypedDict": ".standardjobsubmitresponse",
    "SubmitGraphQlQueryJobResponse": ".submitgraphqlqueryjobop",
    "SubmitGraphQlQueryJobResponseTypedDict": ".submitgraphqlqueryjobop",
    "SubmitGraphQlRequestResponse": ".submitgraphqlrequestop",
    "SubmitGraphQlRequestResponseTypedDict": ".submitgraphqlrequestop",
    "SubmitRestRequestResponse": ".submitrestrequestop",
    "SubmitRestRequestResponseBody": ".submitrestrequestop",
    "SubmitRestRequestResponseBodyTypedDict": ".submitrestrequestop",
    "SubmitRestRequestResponseTypedDict": ".submitrestrequestop",
    "TargetingEndAge": ".targetingendage",
    "TargetingGender": ".targetinggender",
    "TargetingStartAge": ".targetingstartage",
    "ThirdPartyDataInput": ".thirdpartydatainput",
    "ThirdPartyDataInputTypedDict": ".thirdpartydatainput",
    "UpdateAdGroupResponse": ".updateadgroupop",
    "UpdateAdGroupResponseTypedDict": ".updateadgroupop",
    "UpdateAdGroupsJobResponse": ".updateadgroupsjobop",
    "UpdateAdGroupsJobResponseTypedDict": ".updateadgroupsjobop",
    "UpdateCampaignResponse": ".updatecampaignop",
    "UpdateCampaignResponseTypedDict": ".updatecampaignop",
    "UpdateCampaignsJobResponse": ".updatecampaignsjobop",
    "UpdateCampaignsJobResponseTypedDict": ".updatecampaignsjobop",
    "WorkflowCallbackInput": ".workflowcallbackinput",
    "WorkflowCallbackInputTypedDict": ".workflowcallbackinput",
    "WorkflowsError": ".workflowserror",
    "WorkflowStatus": ".workflowstatus",
}


def __getattr__(attr_name: str) -> object:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(
            f"No {attr_name} found in _dynamic_imports for module name -> {__name__} "
        )

    try:
        module = import_module(module_name, __package__)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(
            f"Failed to import {attr_name} from {module_name}: {e}"
        ) from e
    except AttributeError as e:
        raise AttributeError(
            f"Failed to get {attr_name} from {module_name}: {e}"
        ) from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .campaignupdateworkflowprimaryinput import (
    CampaignUpdateWorkflowPrimaryInput,
    CampaignUpdateWorkflowPrimaryInputTypedDict,
)
from .campaignworkflowadvancedinput import (
    CampaignWorkflowAdvancedInput,
    CampaignWorkflowAdvancedInputTypedDict,
)
import pydantic
from pydantic import model_serializer
from ttd_workflows.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CampaignUpdateWorkflowInputWithValidationTypedDict(TypedDict):
    id: Nullable[str]
    primary_input: NotRequired[CampaignUpdateWorkflowPrimaryInputTypedDict]
    advanced_input: NotRequired[CampaignWorkflowAdvancedInputTypedDict]
    validate_input_only: NotRequired[Nullable[bool]]


class CampaignUpdateWorkflowInputWithValidation(BaseModel):
    id: Nullable[str]

    primary_input: Annotated[
        Optional[CampaignUpdateWorkflowPrimaryInput],
        pydantic.Field(alias="primaryInput"),
    ] = None

    advanced_input: Annotated[
        Optional[CampaignWorkflowAdvancedInput], pydantic.Field(alias="advancedInput")
    ] = None

    validate_input_only: Annotated[
        OptionalNullable[bool], pydantic.Field(alias="validateInputOnly")
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["primaryInput", "advancedInput", "validateInputOnly"]
        nullable_fields = ["id", "validateInputOnly"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class EnhancedNielsenReportingOptions(str, Enum):
    NONE = "None"
    AUDIENCE = "Audience"
    SITE = "Site"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from pydantic import model_serializer
from ttd_workflows.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import List
from typing_extensions import Annotated, NotRequired, TypedDict


class AdGroupAudienceTargetingTypedDict(TypedDict):
    audience_accelerator_exclusions_enabled: bool
    audience_booster_enabled: bool
    audience_excluder_enabled: bool
    audience_predictor_enabled: bool
    cross_device_vendor_list_for_audience: Nullable[List[int]]
    recency_exclusion_window_in_minutes: int
    target_trackable_users_enabled: bool
    use_mc_id_as_primary: bool
    audience_id: NotRequired[Nullable[str]]


class AdGroupAudienceTargeting(BaseModel):
    audience_accelerator_exclusions_enabled: Annotated[
        bool, pydantic.Field(alias="audienceAcceleratorExclusionsEnabled")
    ]

    audience_booster_enabled: Annotated[
        bool, pydantic.Field(alias="audienceBoosterEnabled")
    ]

    audience_excluder_enabled: Annotated[
        bool, pydantic.Field(alias="audienceExcluderEnabled")
    ]

    audience_predictor_enabled: Annotated[
        bool, pydantic.Field(alias="audiencePredictorEnabled")
    ]

    cross_device_vendor_list_for_audience: Annotated[
        Nullable[List[int]], pydantic.Field(alias="crossDeviceVendorListForAudience")
    ]

    recency_exclusion_window_in_minutes: Annotated[
        int, pydantic.Field(alias="recencyExclusionWindowInMinutes")
    ]

    target_trackable_users_enabled: Annotated[
        bool, pydantic.Field(alias="targetTrackableUsersEnabled")
    ]

    use_mc_id_as_primary: Annotated[bool, pydantic.Field(alias="useMcIdAsPrimary")]

    audience_id: Annotated[
        OptionalNullable[str], pydantic.Field(alias="audienceId")
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["audienceId"]
        nullable_fields = ["audienceId", "crossDeviceVendorListForAudience"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .campaignversionworkflow import (
    CampaignVersionWorkflow,
    CampaignVersionWorkflowTypedDict,
)
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
import pydantic
from ttd_workflows.types import BaseModel
from ttd_workflows.utils import FieldMetadata, PathParamMetadata
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GetCampaignVersionRequestTypedDict(TypedDict):
    id: str


class GetCampaignVersionRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]


class GetCampaignVersionResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    campaign_version_workflow: NotRequired[CampaignVersionWorkflowTypedDict]
    r"""OK"""


class GetCampaignVersionResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    campaign_version_workflow: Optional[CampaignVersionWorkflow] = None
    r"""OK"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .adgroupchannel import AdGroupChannel
from .adgroupfunnellocation import AdGroupFunnelLocation
from .adgroupworkflowassociatebidlistinput import (
    AdGroupWorkflowAssociateBidListInput,
    AdGroupWorkflowAssociateBidListInputTypedDict,
)
from .adgroupworkflowaudiencetargetinginput import (
    AdGroupWorkflowAudienceTargetingInput,
    AdGroupWorkflowAudienceTargetingInputTypedDict,
)
from .adgroupworkflowbudgetinput import (
    AdGroupWorkflowBudgetInput,
    AdGroupWorkflowBudgetInputTypedDict,
)
from .adgroupworkflowroigoalinput import (
    AdGroupWorkflowROIGoalInput,
    AdGroupWorkflowROIGoalInputTypedDict,
)
import pydantic
from pydantic import model_serializer
from ttd_workflows.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AdGroupCreateWorkflowPrimaryInputTypedDict(TypedDict):
    name: Nullable[str]
    channel: AdGroupChannel
    funnel_location: AdGroupFunnelLocation
    is_enabled: NotRequired[Nullable[bool]]
    description: NotRequired[Nullable[str]]
    budget: NotRequired[AdGroupWorkflowBudgetInputTypedDict]
    base_bid_cpm_in_advertiser_currency: NotRequired[Nullable[float]]
    max_bid_cpm_in_advertiser_currency: NotRequired[Nullable[float]]
    audience_targeting: NotRequired[AdGroupWorkflowAudienceTargetingInputTypedDict]
    roi_goal: NotRequired[AdGroupWorkflowROIGoalInputTypedDict]
    creative_ids: NotRequired[Nullable[List[str]]]
    associated_bid_lists: NotRequired[
        Nullable[List[AdGroupWorkflowAssociateBidListInputTypedDict]]
    ]
    programmatic_guaranteed_private_contract_id: NotRequired[Nullable[str]]


class AdGroupCreateWorkflowPrimaryInput(BaseModel):
    name: Nullable[str]

    channel: AdGroupChannel

    funnel_location: Annotated[
        AdGroupFunnelLocation, pydantic.Field(alias="funnelLocation")
    ]

    is_enabled: Annotated[OptionalNullable[bool], pydantic.Field(alias="isEnabled")] = (
        UNSET
    )

    description: OptionalNullable[str] = UNSET

    budget: Optional[AdGroupWorkflowBudgetInput] = None

    base_bid_cpm_in_advertiser_currency: Annotated[
        OptionalNullable[float], pydantic.Field(alias="baseBidCPMInAdvertiserCurrency")
    ] = UNSET

    max_bid_cpm_in_advertiser_currency: Annotated[
        OptionalNullable[float], pydantic.Field(alias="maxBidCPMInAdvertiserCurrency")
    ] = UNSET

    audience_targeting: Annotated[
        Optional[AdGroupWorkflowAudienceTargetingInput],
        pydantic.Field(alias="audienceTargeting"),
    ] = None

    roi_goal: Annotated[
        Optional[AdGroupWorkflowROIGoalInput], pydantic.Field(alias="roiGoal")
    ] = None

    creative_ids: Annotated[
        OptionalNullable[List[str]], pydantic.Field(alias="creativeIds")
    ] = UNSET

    associated_bid_lists: Annotated[
        OptionalNullable[List[AdGroupWorkflowAssociateBidListInput]],
        pydantic.Field(alias="associatedBidLists"),
    ] = UNSET

    programmatic_guaranteed_private_contract_id: Annotated[
        OptionalNullable[str],
        pydantic.Field(alias="programmaticGuaranteedPrivateContractId"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "isEnabled",
            "description",
            "budget",
            "baseBidCPMInAdvertiserCurrency",
            "maxBidCPMInAdvertiserCurrency",
            "audienceTargeting",
            "roiGoal",
            "creativeIds",
            "associatedBidLists",
            "programmaticGuaranteedPrivateContractId",
        ]
        nullable_fields = [
            "isEnabled",
            "description",
            "baseBidCPMInAdvertiserCurrency",
            "maxBidCPMInAdvertiserCurrency",
            "creativeIds",
            "associatedBidLists",
            "name",
            "programmaticGuaranteedPrivateContractId",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .adgroupcreateworkflowprimaryinput import (
    AdGroupCreateWorkflowPrimaryInput,
    AdGroupCreateWorkflowPrimaryInputTypedDict,
)
from .campaigncreateworkflowadgroupadvancedinput import (
    CampaignCreateWorkflowAdGroupAdvancedInput,
    CampaignCreateWorkflowAdGroupAdvancedInputTypedDict,
)
import pydantic
from ttd_workflows.types import BaseModel
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CampaignCreateWorkflowAdGroupInputTypedDict(TypedDict):
    primary_input: AdGroupCreateWorkflowPrimaryInputTypedDict
    advanced_input: NotRequired[CampaignCreateWorkflowAdGroupAdvancedInputTypedDict]


class CampaignCreateWorkflowAdGroupInput(BaseModel):
    primary_input: Annotated[
        AdGroupCreateWorkflowPrimaryInput, pydantic.Field(alias="primaryInput")
    ]

    advanced_input: Annotated[
        Optional[CampaignCreateWorkflowAdGroupAdvancedInput],
        pydantic.Field(alias="advancedInput"),
    ] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .enhancednielsenreportingoptions import EnhancedNielsenReportingOptions
from .targetingendage import TargetingEndAge
from .targetinggender import TargetingGender
from .targetingstartage import TargetingStartAge
import pydantic
from pydantic import model_serializer
from ttd_workflows.types import BaseModel, Nullable, UNSET_SENTINEL
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AdGroupWorkflowNielsenTrackingAttributesInputTypedDict(TypedDict):
    gender: TargetingGender
    start_age: TargetingStartAge
    end_age: TargetingEndAge
    countries: Nullable[List[str]]
    enhanced_reporting_option: NotRequired[EnhancedNielsenReportingOptions]


class AdGroupWorkflowNielsenTrackingAttributesInput(BaseModel):
    gender: TargetingGender

    start_age: Annotated[TargetingStartAge, pydantic.Field(alias="startAge")]

    end_age: Annotated[TargetingEndAge, pydantic.Field(alias="endAge")]

    countries: Nullable[List[str]]

    enhanced_reporting_option: Annotated[
        Optional[EnhancedNielsenReportingOptions],
        pydantic.Field(alias="enhancedReportingOption"),
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["enhancedReportingOption"]
        nullable_fields = ["countries"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .graphqljobcallbackinput import (
    GraphQlJobCallbackInput,
    GraphQlJobCallbackInputTypedDict,
)
import pydantic
from ttd_workflows.types import BaseModel
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GraphQlQueryJobInputTypedDict(TypedDict):
    r"""Required fields for executing a GraphQL query job"""

    query: str
    r"""The GraphQL query to execute"""
    callback_input: NotRequired[GraphQlJobCallbackInputTypedDict]
    r"""Input class for providing a callback's url and any headers needed for the callback."""


class GraphQlQueryJobInput(BaseModel):
    r"""Required fields for executing a GraphQL query job"""

    query: str
    r"""The GraphQL query to execute"""

    callback_input: Annotated[
        Optional[GraphQlJobCallbackInput], pydantic.Field(alias="callbackInput")
    ] = None
    r"""Input class for providing a callback's url and any headers needed for the callback."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .allocationtype import AllocationType
import pydantic
from pydantic import model_serializer
from ttd_workflows.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CampaignCreateWorkflowAdGroupFlightInputTypedDict(TypedDict):
    allocation_type: NotRequired[AllocationType]
    budget_in_advertiser_currency: NotRequired[Nullable[float]]
    budget_in_impressions: NotRequired[Nullable[int]]
    daily_target_in_advertiser_currency: NotRequired[Nullable[float]]
    daily_target_in_impressions: NotRequired[Nullable[int]]


class CampaignCreateWorkflowAdGroupFlightInput(BaseModel):
    allocation_type: Annotated[
        Optional[AllocationType], pydantic.Field(alias="allocationType")
    ] = None

    budget_in_advertiser_currency: Annotated[
        OptionalNullable[float], pydantic.Field(alias="budgetInAdvertiserCurrency")
    ] = UNSET

    budget_in_impressions: Annotated[
        OptionalNullable[int], pydantic.Field(alias="budgetInImpressions")
    ] = UNSET

    daily_target_in_advertiser_currency: Annotated[
        OptionalNullable[float], pydantic.Field(alias="dailyTargetInAdvertiserCurrency")
    ] = UNSET

    daily_target_in_impressions: Annotated[
        OptionalNullable[int], pydantic.Field(alias="dailyTargetInImpressions")
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "allocationType",
            "budgetInAdvertiserCurrency",
            "budgetInImpressions",
            "dailyTargetInAdvertiserCurrency",
            "dailyTargetInImpressions",
        ]
        nullable_fields = [
            "budgetInAdvertiserCurrency",
            "budgetInImpressions",
            "dailyTargetInAdvertiserCurrency",
            "dailyTargetInImpressions",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from pydantic import model_serializer
from ttd_workflows.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import List
from typing_extensions import Annotated, NotRequired, TypedDict


class AdGroupWorkflowAudienceTargetingInputTypedDict(TypedDict):
    audience_id: NotRequired[Nullable[str]]
    audience_accelerator_exclusions_enabled: NotRequired[Nullable[bool]]
    audience_booster_enabled: NotRequired[Nullable[bool]]
    audience_excluder_enabled: NotRequired[Nullable[bool]]
    audience_predictor_enabled: NotRequired[Nullable[bool]]
    cross_device_vendor_list_for_audience: NotRequired[Nullable[List[int]]]
    recency_exclusion_window_in_minutes: NotRequired[Nullable[int]]
    target_trackable_users_enabled: NotRequired[Nullable[bool]]
    use_mc_id_as_primary: NotRequired[Nullable[bool]]


class AdGroupWorkflowAudienceTargetingInput(BaseModel):
    audience_id: Annotated[
        OptionalNullable[str], pydantic.Field(alias="audienceId")
    ] = UNSET

    audience_accelerator_exclusions_enabled: Annotated[
        OptionalNullable[bool],
        pydantic.Field(alias="audienceAcceleratorExclusionsEnabled"),
    ] = UNSET

    audience_booster_enabled: Annotated[
        OptionalNullable[bool], pydantic.Field(alias="audienceBoosterEnabled")
    ] = UNSET

    audience_excluder_enabled: Annotated[
        OptionalNullable[bool], pydantic.Field(alias="audienceExcluderEnabled")
    ] = UNSET

    audience_predictor_enabled: Annotated[
        OptionalNullable[bool], pydantic.Field(alias="audiencePredictorEnabled")
    ] = UNSET

    cross_device_vendor_list_for_audience: Annotated[
        OptionalNullable[List[int]],
        pydantic.Field(alias="crossDeviceVendorListForAudience"),
    ] = UNSET

    recency_exclusion_window_in_minutes: Annotated[
        OptionalNullable[int], pydantic.Field(alias="recencyExclusionWindowInMinutes")
    ] = UNSET

    target_trackable_users_enabled: Annotated[
        OptionalNullable[bool], pydantic.Field(alias="targetTrackableUsersEnabled")
    ] = UNSET

    use_mc_id_as_primary: Annotated[
        OptionalNullable[bool], pydantic.Field(alias="useMcIdAsPrimary")
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "audienceId",
            "audienceAcceleratorExclusionsEnabled",
            "audienceBoosterEnabled",
            "audienceExcluderEnabled",
            "audiencePredictorEnabled",
            "crossDeviceVendorListForAudience",
            "recencyExclusionWindowInMinutes",
            "targetTrackableUsersEnabled",
            "useMcIdAsPrimary",
        ]
        nullable_fields = [
            "audienceId",
            "audienceAcceleratorExclusionsEnabled",
            "audienceBoosterEnabled",
            "audienceExcluderEnabled",
            "audiencePredictorEnabled",
            "crossDeviceVendorListForAudience",
            "recencyExclusionWindowInMinutes",
            "targetTrackableUsersEnabled",
            "useMcIdAsPrimary",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class TargetingStartAge(str, Enum):
    TWO = "Two"
    THIRTEEN = "Thirteen"
    EIGHTEEN = "Eighteen"
    TWENTY_ONE = "TwentyOne"
    TWENTY_FIVE = "TwentyFive"
    THIRTY = "Thirty"
    THIRTY_FIVE = "ThirtyFive"
    FORTY = "Forty"
    FORTY_FIVE = "FortyFive"
    FIFTY = "Fifty"
    FIFTY_FIVE = "FiftyFive"
    SIXTY_FIVE = "SixtyFive"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from ttd_workflows.types import BaseModel
from typing_extensions import Annotated, TypedDict


class AdGroupContractTargetingTypedDict(TypedDict):
    allow_open_market_bidding_when_targeting_contracts: bool


class AdGroupContractTargeting(BaseModel):
    allow_open_market_bidding_when_targeting_contracts: Annotated[
        bool, pydantic.Field(alias="allowOpenMarketBiddingWhenTargetingContracts")
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .adgroupworkflowcomscoresettingsinput import (
    AdGroupWorkflowComscoreSettingsInput,
    AdGroupWorkflowComscoreSettingsInputTypedDict,
)
from .adgroupworkflowcontracttargetinginput import (
    AdGroupWorkflowContractTargetingInput,
    AdGroupWorkflowContractTargetingInputTypedDict,
)
from .adgroupworkflowkoaoptimizationsettingsinput import (
    AdGroupWorkflowKoaOptimizationSettingsInput,
    AdGroupWorkflowKoaOptimizationSettingsInputTypedDict,
)
from .adgroupworkflownewfrequencyconfiginput import (
    AdGroupWorkflowNewFrequencyConfigInput,
    AdGroupWorkflowNewFrequencyConfigInputTypedDict,
)
from .adgroupworkflownielsentrackingattributesinput import (
    AdGroupWorkflowNielsenTrackingAttributesInput,
    AdGroupWorkflowNielsenTrackingAttributesInputTypedDict,
)
from .campaigncreateworkflowadgroupflightinput import (
    CampaignCreateWorkflowAdGroupFlightInput,
    CampaignCreateWorkflowAdGroupFlightInputTypedDict,
)
from .dimensionalbiddingdimensions import DimensionalBiddingDimensions
import pydantic
from pydantic import model_serializer
from ttd_workflows.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CampaignCreateWorkflowAdGroupAdvancedInputTypedDict(TypedDict):
    koa_optimization_settings: NotRequired[
        AdGroupWorkflowKoaOptimizationSettingsInputTypedDict
    ]
    comscore_settings: NotRequired[AdGroupWorkflowComscoreSettingsInputTypedDict]
    contract_targeting: NotRequired[AdGroupWorkflowContractTargetingInputTypedDict]
    dimensional_bidding_auto_optimization_settings: NotRequired[
        Nullable[List[List[DimensionalBiddingDimensions]]]
    ]
    is_use_clicks_as_conversions_enabled: NotRequired[Nullable[bool]]
    is_use_secondary_conversions_enabled: NotRequired[Nullable[bool]]
    nielsen_tracking_attributes: NotRequired[
        AdGroupWorkflowNielsenTrackingAttributesInputTypedDict
    ]
    new_frequency_configs: NotRequired[
        Nullable[List[AdGroupWorkflowNewFrequencyConfigInputTypedDict]]
    ]
    flights: NotRequired[
        Nullable[List[CampaignCreateWorkflowAdGroupFlightInputTypedDict]]
    ]


class CampaignCreateWorkflowAdGroupAdvancedInput(BaseModel):
    koa_optimization_settings: Annotated[
        Optional[AdGroupWorkflowKoaOptimizationSettingsInput],
        pydantic.Field(alias="koaOptimizationSettings"),
    ] = None

    comscore_settings: Annotated[
        Optional[AdGroupWorkflowComscoreSettingsInput],
        pydantic.Field(alias="comscoreSettings"),
    ] = None

    contract_targeting: Annotated[
        Optional[AdGroupWorkflowContractTargetingInput],
        pydantic.Field(alias="contractTargeting"),
    ] = None

    dimensional_bidding_auto_optimization_settings: Annotated[
        OptionalNullable[List[List[DimensionalBiddingDimensions]]],
        pydantic.Field(alias="dimensionalBiddingAutoOptimizationSettings"),
    ] = UNSET

    is_use_clicks_as_conversions_enabled: Annotated[
        OptionalNullable[bool], pydantic.Field(alias="isUseClicksAsConversionsEnabled")
    ] = UNSET

    is_use_secondary_conversions_enabled: Annotated[
        OptionalNullable[bool], pydantic.Field(alias="isUseSecondaryConversionsEnabled")
    ] = UNSET

    nielsen_tracking_attributes: Annotated[
        Optional[AdGroupWorkflowNielsenTrackingAttributesInput],
        pydantic.Field(alias="nielsenTrackingAttributes"),
    ] = None

    new_frequency_configs: Annotated[
        OptionalNullable[List[AdGroupWorkflowNewFrequencyConfigInput]],
        pydantic.Field(alias="newFrequencyConfigs"),
    ] = UNSET

    flights: OptionalNullable[List[CampaignCreateWorkflowAdGroupFlightInput]] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "koaOptimizationSettings",
            "comscoreSettings",
            "contractTargeting",
            "dimensionalBiddingAutoOptimizationSettings",
            "isUseClicksAsConversionsEnabled",
            "isUseSecondaryConversionsEnabled",
            "nielsenTrackingAttributes",
            "newFrequencyConfigs",
            "flights",
        ]
        nullable_fields = [
            "dimensionalBiddingAutoOptimizationSettings",
            "isUseClicksAsConversionsEnabled",
            "isUseSecondaryConversionsEnabled",
            "newFrequencyConfigs",
            "flights",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class WorkflowStatus(str, Enum):
    IN_PROGRESS = "InProgress"
    FAILURE = "Failure"
    SUCCESS = "Success"
    PARTIAL_SUCCESS = "PartialSuccess"
    RESULTS_EXPIRED = "ResultsExpired"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .adgroupworkflow import AdGroupWorkflow, AdGroupWorkflowTypedDict
import pydantic
from ttd_workflows.types import BaseModel
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AdGroupPayloadTypedDict(TypedDict):
    ad_group: NotRequired[AdGroupWorkflowTypedDict]


class AdGroupPayload(BaseModel):
    ad_group: Annotated[Optional[AdGroupWorkflow], pydantic.Field(alias="adGroup")] = (
        None
    )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
from .standardjobstatusresponse import (
    StandardJobStatusResponse,
    StandardJobStatusResponseTypedDict,
)
import pydantic
from ttd_workflows.types import BaseModel
from ttd_workflows.utils import FieldMetadata, PathParamMetadata
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GetJobStatusRequestTypedDict(TypedDict):
    id: int


class GetJobStatusRequest(BaseModel):
    id: Annotated[
        int, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]


class GetJobStatusResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    standard_job_status_response: NotRequired[StandardJobStatusResponseTypedDict]
    r"""OK"""


class GetJobStatusResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    standard_job_status_response: Optional[StandardJobStatusResponse] = None
    r"""OK"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .adgroupassociatebidlist import (
    AdGroupAssociateBidList,
    AdGroupAssociateBidListTypedDict,
)
from .adgroupaudiencetargeting import (
    AdGroupAudienceTargeting,
    AdGroupAudienceTargetingTypedDict,
)
from .adgroupbudget import AdGroupBudget, AdGroupBudgetTypedDict
from .adgroupchannel import AdGroupChannel
from .adgroupcomscoresettings import (
    AdGroupComscoreSettings,
    AdGroupComscoreSettingsTypedDict,
)
from .adgroupcontracttargeting import (
    AdGroupContractTargeting,
    AdGroupContractTargetingTypedDict,
)
from .adgroupflight import AdGroupFlight, AdGroupFlightTypedDict
from .adgroupfunnellocation import AdGroupFunnelLocation
from .adgroupkoaoptimizationsettings import (
    AdGroupKoaOptimizationSettings,
    AdGroupKoaOptimizationSettingsTypedDict,
)
from .adgroupnielsentrackingattributes import (
    AdGroupNielsenTrackingAttributes,
    AdGroupNielsenTrackingAttributesTypedDict,
)
from .adgrouproigoal import AdGroupROIGoal, AdGroupROIGoalTypedDict
from .dimensionalbiddingdimensions import DimensionalBiddingDimensions
import pydantic
from pydantic import model_serializer
from ttd_workflows.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AdGroupWorkflowTypedDict(TypedDict):
    id: Nullable[str]
    name: Nullable[str]
    campaign_id: Nullable[str]
    is_enabled: bool
    channel: AdGroupChannel
    funnel_location: AdGroupFunnelLocation
    budget: AdGroupBudgetTypedDict
    roi_goal: AdGroupROIGoalTypedDict
    creative_ids: Nullable[List[str]]
    associated_bid_lists: Nullable[List[AdGroupAssociateBidListTypedDict]]
    flights: Nullable[List[AdGroupFlightTypedDict]]
    comscore_settings: AdGroupComscoreSettingsTypedDict
    contract_targeting: AdGroupContractTargetingTypedDict
    dimensional_bidding_auto_optimization_settings: Nullable[
        List[List[DimensionalBiddingDimensions]]
    ]
    is_use_clicks_as_conversions_enabled: bool
    is_use_secondary_conversions_enabled: bool
    description: NotRequired[Nullable[str]]
    programmatic_guaranteed_private_contract_id: NotRequired[Nullable[str]]
    base_bid_cpm_in_advertiser_currency: NotRequired[Nullable[float]]
    max_bid_cpm_in_advertiser_currency: NotRequired[Nullable[float]]
    audience_targeting: NotRequired[AdGroupAudienceTargetingTypedDict]
    koa_optimization_settings: NotRequired[AdGroupKoaOptimizationSettingsTypedDict]
    nielsen_tracking_attributes: NotRequired[AdGroupNielsenTrackingAttributesTypedDict]


class AdGroupWorkflow(BaseModel):
    id: Nullable[str]

    name: Nullable[str]

    campaign_id: Annotated[Nullable[str], pydantic.Field(alias="campaignId")]

    is_enabled: Annotated[bool, pydantic.Field(alias="isEnabled")]

    channel: AdGroupChannel

    funnel_location: Annotated[
        AdGroupFunnelLocation, pydantic.Field(alias="funnelLocation")
    ]

    budget: AdGroupBudget

    roi_goal: Annotated[AdGroupROIGoal, pydantic.Field(alias="roiGoal")]

    creative_ids: Annotated[Nullable[List[str]], pydantic.Field(alias="creativeIds")]

    associated_bid_lists: Annotated[
        Nullable[List[AdGroupAssociateBidList]],
        pydantic.Field(alias="associatedBidLists"),
    ]

    flights: Nullable[List[AdGroupFlight]]

    comscore_settings: Annotated[
        AdGroupComscoreSettings, pydantic.Field(alias="comscoreSettings")
    ]

    contract_targeting: Annotated[
        AdGroupContractTargeting, pydantic.Field(alias="contractTargeting")
    ]

    dimensional_bidding_auto_optimization_settings: Annotated[
        Nullable[List[List[DimensionalBiddingDimensions]]],
        pydantic.Field(alias="dimensionalBiddingAutoOptimizationSettings"),
    ]

    is_use_clicks_as_conversions_enabled: Annotated[
        bool, pydantic.Field(alias="isUseClicksAsConversionsEnabled")
    ]

    is_use_secondary_conversions_enabled: Annotated[
        bool, pydantic.Field(alias="isUseSecondaryConversionsEnabled")
    ]

    description: OptionalNullable[str] = UNSET

    programmatic_guaranteed_private_contract_id: Annotated[
        OptionalNullable[str],
        pydantic.Field(alias="programmaticGuaranteedPrivateContractId"),
    ] = UNSET

    base_bid_cpm_in_advertiser_currency: Annotated[
        OptionalNullable[float], pydantic.Field(alias="baseBidCPMInAdvertiserCurrency")
    ] = UNSET

    max_bid_cpm_in_advertiser_currency: Annotated[
        OptionalNullable[float], pydantic.Field(alias="maxBidCPMInAdvertiserCurrency")
    ] = UNSET

    audience_targeting: Annotated[
        Optional[AdGroupAudienceTargeting], pydantic.Field(alias="audienceTargeting")
    ] = None

    koa_optimization_settings: Annotated[
        Optional[AdGroupKoaOptimizationSettings],
        pydantic.Field(alias="koaOptimizationSettings"),
    ] = None

    nielsen_tracking_attributes: Annotated[
        Optional[AdGroupNielsenTrackingAttributes],
        pydantic.Field(alias="nielsenTrackingAttributes"),
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "description",
            "programmaticGuaranteedPrivateContractId",
            "baseBidCPMInAdvertiserCurrency",
            "maxBidCPMInAdvertiserCurrency",
            "audienceTargeting",
            "koaOptimizationSettings",
            "nielsenTrackingAttributes",
        ]
        nullable_fields = [
            "id",
            "name",
            "campaignId",
            "description",
            "programmaticGuaranteedPrivateContractId",
            "baseBidCPMInAdvertiserCurrency",
            "maxBidCPMInAdvertiserCurrency",
            "creativeIds",
            "associatedBidLists",
            "flights",
            "dimensionalBiddingAutoOptimizationSettings",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .campaignchanneltype import CampaignChannelType
from .campaignflightworkflow import (
    CampaignFlightWorkflow,
    CampaignFlightWorkflowTypedDict,
)
from .customcpatype import CustomCPAType
from datetime import datetime
import pydantic
from pydantic import model_serializer
from ttd_workflows.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CampaignWorkflowTypedDict(TypedDict):
    id: Nullable[str]
    name: NotRequired[Nullable[str]]
    description: NotRequired[Nullable[str]]
    start_date: NotRequired[Nullable[datetime]]
    end_date: NotRequired[Nullable[datetime]]
    time_zone: NotRequired[Nullable[str]]
    custom_cpa_click_weight: NotRequired[Nullable[float]]
    custom_cpa_viewthrough_weight: NotRequired[Nullable[float]]
    custom_cpa_type: NotRequired[CustomCPAType]
    impressions_only_budgeting_cpm: NotRequired[Nullable[float]]
    primary_channel: NotRequired[CampaignChannelType]
    primary_goal: NotRequired[Nullable[str]]
    seed_id: NotRequired[Nullable[str]]
    conversion_reporting_columns_count: NotRequired[Nullable[int]]
    flights: NotRequired[Nullable[List[CampaignFlightWorkflowTypedDict]]]


class CampaignWorkflow(BaseModel):
    id: Nullable[str]

    name: OptionalNullable[str] = UNSET

    description: OptionalNullable[str] = UNSET

    start_date: Annotated[
        OptionalNullable[datetime], pydantic.Field(alias="startDate")
    ] = UNSET

    end_date: Annotated[OptionalNullable[datetime], pydantic.Field(alias="endDate")] = (
        UNSET
    )

    time_zone: Annotated[OptionalNullable[str], pydantic.Field(alias="timeZone")] = (
        UNSET
    )

    custom_cpa_click_weight: Annotated[
        OptionalNullable[float], pydantic.Field(alias="customCPAClickWeight")
    ] = UNSET

    custom_cpa_viewthrough_weight: Annotated[
        OptionalNullable[float], pydantic.Field(alias="customCPAViewthroughWeight")
    ] = UNSET

    custom_cpa_type: Annotated[
        Optional[CustomCPAType], pydantic.Field(alias="customCPAType")
    ] = None

    impressions_only_budgeting_cpm: Annotated[
        OptionalNullable[float], pydantic.Field(alias="impressionsOnlyBudgetingCpm")
    ] = UNSET

    primary_channel: Annotated[
        Optional[CampaignChannelType], pydantic.Field(alias="primaryChannel")
    ] = None

    primary_goal: Annotated[
        OptionalNullable[str], pydantic.Field(alias="primaryGoal")
    ] = UNSET

    seed_id: Annotated[OptionalNullable[str], pydantic.Field(alias="seedId")] = UNSET

    conversion_reporting_columns_count: Annotated[
        OptionalNullable[int], pydantic.Field(alias="conversionReportingColumnsCount")
    ] = UNSET

    flights: OptionalNullable[List[CampaignFlightWorkflow]] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "name",
            "description",
            "startDate",
            "endDate",
            "timeZone",
            "customCPAClickWeight",
            "customCPAViewthroughWeight",
            "customCPAType",
            "impressionsOnlyBudgetingCpm",
            "primaryChannel",
            "primaryGoal",
            "seedId",
            "conversionReportingColumnsCount",
            "flights",
        ]
        nullable_fields = [
            "id",
            "name",
            "description",
            "startDate",
            "endDate",
            "timeZone",
            "customCPAClickWeight",
            "customCPAViewthroughWeight",
            "impressionsOnlyBudgetingCpm",
            "primaryGoal",
            "seedId",
            "conversionReportingColumnsCount",
            "flights",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class DimensionalBiddingDimensions(str, Enum):
    UNKNOWN_DIMENSION = "UnknownDimension"
    HAS_AD_FORMAT_ID = "HasAdFormatId"
    HAS_DOMAIN_FRAGMENT_ID = "HasDomainFragmentId"
    HAS_UNIVERSAL_CATEGORY_TAXONOMY_ID = "HasUniversalCategoryTaxonomyId"
    HAS_PLACEMENT_POSITION_RELATIVE_TO_FOLD_ID = "HasPlacementPositionRelativeToFoldId"
    HAS_ADVERTISER_TARGETING_DATA_ID = "HasAdvertiserTargetingDataId"
    HAS_BROWSER_ID = "HasBrowserId"
    HAS_CARRIER_ID = "HasCarrierId"
    HAS_DEVICE_MODEL_ID = "HasDeviceModelId"
    HAS_DEVICE_TYPE_ID = "HasDeviceTypeId"
    HAS_GEO_SEGMENT_ID = "HasGeoSegmentId"
    HAS_HOUR_OF_WEEK = "HasHourOfWeek"
    HAS_LANGUAGE_ID = "HasLanguageId"
    HAS_OS_ID = "HasOsId"
    HAS_PRIVATE_CONTRACT_ID = "HasPrivateContractId"
    HAS_RECENCY_WINDOW_IN_MINUTES = "HasRecencyWindowInMinutes"
    HAS_RENDERING_CONTEXT_ID = "HasRenderingContextId"
    HAS_FACTUAL_GEOSTORE_TARGETING_DATA_ID = "HasFactualGeostoreTargetingDataId"
    HAS_SUPPLY_VENDOR_ID = "HasSupplyVendorId"
    HAS_TEMPERATURE_BUCKET_IN_CELSIUS = "HasTemperatureBucketInCelsius"
    HAS_THIRD_PARTY_DATA = "HasThirdPartyData"
    HAS_ADS_TXT_SELLER_TYPE_ID = "HasAdsTxtSellerTypeId"
    HAS_VIDEO_MUTED_STATE_ID = "HasVideoMutedStateId"
    HAS_VIDEO_PLAYBACK_TYPE_ID = "HasVideoPlaybackTypeId"
    HAS_VIDEO_PLAYER_SIZE_ID = "HasVideoPlayerSizeId"
    HAS_VIDEO_QUALITY_ID = "HasVideoQualityId"
    HAS_VIDEO_SKIPPABILITY_ID = "HasVideoSkippabilityId"
    HAS_FACTUAL_PROXIMITY_DATA_ID = "HasFactualProximityDataId"
    HAS_IMPRESSION_PLACEMENT_ID = "HasImpressionPlacementId"
    HAS_VIDEO_TRUTH_SCORE_RANGE = "HasVideoTruthScoreRange"
    HAS_VIDEO_VIEWABILITY_SCORE_RANGE = "HasVideoViewabilityScoreRange"
    HAS_DISPLAY_VIEWABILITY_SCORE_RANGE = "HasDisplayViewabilityScoreRange"
    HAS_PEER39_LANGUAGE_ID = "HasPeer39LanguageId"
    HAS_INTEGRAL_CONTEXTUAL_CATEGORY_ID = "HasIntegralContextualCategoryId"
    HAS_PEER39_CONTEXTUAL_CATEGORY_ID = "HasPeer39ContextualCategoryId"
    HAS_DOUBLE_VERIFY_CONTEXTUAL_CATEGORY_ID = "HasDoubleVerifyContextualCategoryId"
    HAS_INTEGRAL_BRAND_SAFETY_CATEGORY_ID = "HasIntegralBrandSafetyCategoryId"
    HAS_INTEGRAL_VIDEO_BRAND_SAFETY_CATEGORY_ID = (
        "HasIntegralVideoBrandSafetyCategoryId"
    )
    HAS_PEER39_BRAND_SAFETY_CATEGORY_ID = "HasPeer39BrandSafetyCategoryId"
    HAS_DOUBLE_VERIFY_BRAND_SAFETY_CATEGORY_ID = "HasDoubleVerifyBrandSafetyCategoryId"
    HAS_INTEGRAL_PAGE_QUALITY_CATEGORY_ID = "HasIntegralPageQualityCategoryId"
    HAS_INTEGRAL_VIDEO_PAGE_QUALITY_CATEGORY_ID = (
        "HasIntegralVideoPageQualityCategoryId"
    )
    HAS_PEER39_PAGE_QUALITY_CATEGORY_ID = "HasPeer39PageQualityCategoryId"
    HAS_DOUBLE_VERIFY_BOT_AVOIDANCE_CATEGORY_ID = (
        "HasDoubleVerifyBotAvoidanceCategoryId"
    )
    HAS_CONTEXTUAL_APP_CATEGORY_ID = "HasContextualAppCategoryId"
    HAS_PUBLISHER_ID = "HasPublisherId"
    HAS_INTEGRAL_VIEWABILITY_CATEGORY_ID = "HasIntegralViewabilityCategoryId"
    HAS_INTEGRAL_VIDEO_VIEWABILITY_CATEGORY_ID = "HasIntegralVideoViewabilityCategoryId"
    HAS_DOUBLE_VERIFY_VIEWABILITY_CATEGORY_ID = "HasDoubleVerifyViewabilityCategoryId"
    HAS_DOUBLE_VERIFY_VIDEO_VIEWABILITY_CATEGORY_ID = (
        "HasDoubleVerifyVideoViewabilityCategoryId"
    )
    HAS_FREQUENCY_ADJUSTMENT_ID = "HasFrequencyAdjustmentId"
    HAS_PEER39_VIEWABILITY_CATEGORY_ID = "HasPeer39ViewabilityCategoryId"
    HAS_VIDEO_COMPLETION_RATE_SCORE_RANGE = "HasVideoCompletionRateScoreRange"
    HAS_INTERNET_CONNECTION_TYPE_ID = "HasInternetConnectionTypeId"
    HAS_AD_BUG_PAGE_QUALITY_CATEGORY_ID = "HasAdBugPageQualityCategoryId"
    HAS_AD_BUG_VIDEO_PAGE_QUALITY_CATEGORY_ID = "HasAdBugVideoPageQualityCategoryId"
    HAS_RTB_ASIA_PAGE_QUALITY_CATEGORY_ID = "HasRTBAsiaPageQualityCategoryId"
    HAS_RTB_ASIA_VIDEO_PAGE_QUALITY_CATEGORY_ID = "HasRTBAsiaVideoPageQualityCategoryId"
    HAS_TTD_CONTEXTUAL_CATEGORY_ID = "HasTTDContextualCategoryId"
    HAS_SUPPLY_PATH_OPTIMIZATION_LIST_ID = "HasSupplyPathOptimizationListId"
    HAS_MARKET_TYPE_ID = "HasMarketTypeId"
    HAS_CONTEXTUAL_SUPPLY_VENDOR_GENRE_ID = "HasContextualSupplyVendorGenreId"
    HAS_CONTEXTUAL_SUPPLY_VENDOR_CATEGORY_ID = "HasContextualSupplyVendorCategoryId"
    HAS_WEATHER_CONDITION_ID = "HasWeatherConditionId"
    HAS_FREQUENCY_V2_ADJUSTMENT_ID = "HasFrequencyV2AdjustmentId"
    HAS_SEMASIO_CONTEXTUAL_CATEGORY_ID = "HasSemasioContextualCategoryId"
    HAS_SELLER_ID = "HasSellerId"
    HAS_DIGITAL_OUT_OF_HOME_VENUE_TYPE_ID = "HasDigitalOutOfHomeVenueTypeId"
    HAS_NATIVE_PLACEMENT_TYPE_ID = "HasNativePlacementTypeId"
    HAS_TTD_NATIVE_CONTEXT_TYPE_ID = "HasTTDNativeContextTypeId"
    HAS_DOMAIN_CLASS_ID = "HasDomainClassId"
    HAS_GEOFENCE_THIRD_PARTY_DATA_AND_BRAND = "HasGeofenceThirdPartyDataAndBrand"
    HAS_PRE_BID_CONTEXTUAL_CATEGORY_ID = "HasPreBidContextualCategoryId"
    HAS_MIAOZHEN_PAGE_QUALITY_CATEGORY_ID = "HasMiaozhenPageQualityCategoryId"
    HAS_MIAOZHEN_VIDEO_PAGE_QUALITY_CATEGORY_ID = (
        "HasMiaozhenVideoPageQualityCategoryId"
    )
    HAS_TENCENT_PAGE_QUALITY_CATEGORY_ID = "HasTencentPageQualityCategoryId"
    HAS_CONTENT_GENRE = "HasContentGenre"
    HAS_CONTENT_LIVESTREAM = "HasContentLivestream"
    HAS_CONTENT_PRODUCTION_QUALITY = "HasContentProductionQuality"
    HAS_CONTENT_RATING = "HasContentRating"
    HAS_CONTENT_DURATION = "HasContentDuration"
    HAS_TV_QUALITY_INDEX_RANGE = "HasTVQualityIndexRange"
    HAS_ATMOSPHERIC_CONDITION = "HasAtmosphericCondition"
    HAS_MARKETPLACE_ID = "HasMarketplaceId"
    HAS_AUDIENCE_REACH_PERCENTAGE_TIER_ID = "HasAudienceReachPercentageTierId"
    HAS_AD_LOAD_CATEGORY = "HasAdLoadCategory"
    HAS_FULL_REFERRER_URL = "HasFullReferrerUrl"
    HAS_LIKELY_REFRESH_RATE = "HasLikelyRefreshRate"
    HAS_CONTENT_TRANSPARENCY = "HasContentTransparency"
    HAS_IDIOSYNCRATIC_SEGMENT_ID = "HasIdiosyncraticSegmentId"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from pydantic import model_serializer
from ttd_workflows.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing_extensions import Annotated, NotRequired, TypedDict


class CampaignWorkflowROIGoalInputTypedDict(TypedDict):
    maximize_reach: NotRequired[Nullable[bool]]
    maximize_ltv_incremental_reach: NotRequired[Nullable[bool]]
    cpc_in_advertiser_currency: NotRequired[Nullable[float]]
    ctr_in_percent: NotRequired[Nullable[float]]
    nielsen_otp_in_percent: NotRequired[Nullable[float]]
    cpa_in_advertiser_currency: NotRequired[Nullable[float]]
    return_on_ad_spend_percent: NotRequired[Nullable[float]]
    vcr_in_percent: NotRequired[Nullable[float]]
    viewability_in_percent: NotRequired[Nullable[float]]
    vcpm_in_advertiser_currency: NotRequired[Nullable[float]]
    cpcv_in_advertiser_currency: NotRequired[Nullable[float]]
    miaozhen_otp_in_percent: NotRequired[Nullable[float]]


class CampaignWorkflowROIGoalInput(BaseModel):
    maximize_reach: Annotated[
        OptionalNullable[bool], pydantic.Field(alias="maximizeReach")
    ] = UNSET

    maximize_ltv_incremental_reach: Annotated[
        OptionalNullable[bool], pydantic.Field(alias="maximizeLtvIncrementalReach")
    ] = UNSET

    cpc_in_advertiser_currency: Annotated[
        OptionalNullable[float], pydantic.Field(alias="cpcInAdvertiserCurrency")
    ] = UNSET

    ctr_in_percent: Annotated[
        OptionalNullable[float], pydantic.Field(alias="ctrInPercent")
    ] = UNSET

    nielsen_otp_in_percent: Annotated[
        OptionalNullable[float], pydantic.Field(alias="nielsenOTPInPercent")
    ] = UNSET

    cpa_in_advertiser_currency: Annotated[
        OptionalNullable[float], pydantic.Field(alias="cpaInAdvertiserCurrency")
    ] = UNSET

    return_on_ad_spend_percent: Annotated[
        OptionalNullable[float], pydantic.Field(alias="returnOnAdSpendPercent")
    ] = UNSET

    vcr_in_percent: Annotated[
        OptionalNullable[float], pydantic.Field(alias="vcrInPercent")
    ] = UNSET

    viewability_in_percent: Annotated[
        OptionalNullable[float], pydantic.Field(alias="viewabilityInPercent")
    ] = UNSET

    vcpm_in_advertiser_currency: Annotated[
        OptionalNullable[float], pydantic.Field(alias="vcpmInAdvertiserCurrency")
    ] = UNSET

    cpcv_in_advertiser_currency: Annotated[
        OptionalNullable[float], pydantic.Field(alias="cpcvInAdvertiserCurrency")
    ] = UNSET

    miaozhen_otp_in_percent: Annotated[
        OptionalNullable[float], pydantic.Field(alias="miaozhenOTPInPercent")
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "maximizeReach",
            "maximizeLtvIncrementalReach",
            "cpcInAdvertiserCurrency",
            "ctrInPercent",
            "nielsenOTPInPercent",
            "cpaInAdvertiserCurrency",
            "returnOnAdSpendPercent",
            "vcrInPercent",
            "viewabilityInPercent",
            "vcpmInAdvertiserCurrency",
            "cpcvInAdvertiserCurrency",
            "miaozhenOTPInPercent",
        ]
        nullable_fields = [
            "maximizeReach",
            "maximizeLtvIncrementalReach",
            "cpcInAdvertiserCurrency",
            "ctrInPercent",
            "nielsenOTPInPercent",
            "cpaInAdvertiserCurrency",
            "returnOnAdSpendPercent",
            "vcrInPercent",
            "viewabilityInPercent",
            "vcpmInAdvertiserCurrency",
            "cpcvInAdvertiserCurrency",
            "miaozhenOTPInPercent",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from ttd_workflows import models, utils
from ttd_workflows._hooks import HookContext
from ttd_workflows.types import BaseModel, OptionalNullable, UNSET
from ttd_workflows.utils import get_security_from_env
from ttd_workflows.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Mapping, Optional, Union, cast


class AdGroup(BaseSDK):
    def create_ad_groups_job(
        self,
        *,
        request: Optional[
            Union[
                models.AdGroupBulkCreateWorkflowInputWithValidation,
                models.AdGroupBulkCreateWorkflowInputWithValidationTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CreateAdGroupsJobResponse:
        r"""Create multiple new ad groups with required fields

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(
                request, Optional[models.AdGroupBulkCreateWorkflowInputWithValidation]
            )
        request = cast(
            Optional[models.AdGroupBulkCreateWorkflowInputWithValidation], request
        )

        req = self._build_request(
            method="POST",
            path="/standardjob/adgroup/bulk",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                True,
                "json",
                Optional[models.AdGroupBulkCreateWorkflowInputWithValidation],
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createAdGroupsJob",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "403", "4XX", "500", "503", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "202", "application/json"):
            return models.CreateAdGroupsJobResponse(
                standard_job_submit_response=unmarshal_json_response(
                    Optional[models.StandardJobSubmitResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, ["400", "403"], "application/json"):
            response_data = unmarshal_json_response(
                models.ProblemDetailsErrorData, http_res
            )
            raise models.ProblemDetailsError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "503", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def create_ad_groups_job_async(
        self,
        *,
        request: Optional[
            Union[
                models.AdGroupBulkCreateWorkflowInputWithValidation,
                models.AdGroupBulkCreateWorkflowInputWithValidationTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CreateAdGroupsJobResponse:
        r"""Create multiple new ad groups with required fields

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(
                request, Optional[models.AdGroupBulkCreateWorkflowInputWithValidation]
            )
        request = cast(
            Optional[models.AdGroupBulkCreateWorkflowInputWithValidation], request
        )

        req = self._build_request_async(
            method="POST",
            path="/standardjob/adgroup/bulk",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                True,
                "json",
                Optional[models.AdGroupBulkCreateWorkflowInputWithValidation],
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createAdGroupsJob",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "403", "4XX", "500", "503", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "202", "application/json"):
            return models.CreateAdGroupsJobResponse(
                standard_job_submit_response=unmarshal_json_response(
                    Optional[models.StandardJobSubmitResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, ["400", "403"], "application/json"):
            response_data = unmarshal_json_response(
                models.ProblemDetailsErrorData, http_res
            )
            raise models.ProblemDetailsError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "503", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def update_ad_groups_job(
        self,
        *,
        request: Optional[
            Union[
                models.AdGroupBulkUpdateWorkflowInputWithValidation,
                models.AdGroupBulkUpdateWorkflowInputWithValidationTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.UpdateAdGroupsJobResponse:
        r"""Update multiple ad groups with specified fields

        Only the fields provided in the request payload for each specific ad group will be updated.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(
                request, Optional[models.AdGroupBulkUpdateWorkflowInputWithValidation]
            )
        request = cast(
            Optional[models.AdGroupBulkUpdateWorkflowInputWithValidation], request
        )

        req = self._build_request(
            method="PATCH",
            path="/standardjob/adgroup/bulk",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                True,
                "json",
                Optional[models.AdGroupBulkUpdateWorkflowInputWithValidation],
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateAdGroupsJob",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "403", "4XX", "500", "503", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "202", "application/json"):
            return models.UpdateAdGroupsJobResponse(
                standard_job_submit_response=unmarshal_json_response(
                    Optional[models.StandardJobSubmitResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, ["400", "403"], "application/json"):
            response_data = unmarshal_json_response(
                models.ProblemDetailsErrorData, http_res
            )
            raise models.ProblemDetailsError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "503", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def update_ad_groups_job_async(
        self,
        *,
        request: Optional[
            Union[
                models.AdGroupBulkUpdateWorkflowInputWithValidation,
                models.AdGroupBulkUpdateWorkflowInputWithValidationTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.UpdateAdGroupsJobResponse:
        r"""Update multiple ad groups with specified fields

        Only the fields provided in the request payload for each specific ad group will be updated.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(
                request, Optional[models.AdGroupBulkUpdateWorkflowInputWithValidation]
            )
        request = cast(
            Optional[models.AdGroupBulkUpdateWorkflowInputWithValidation], request
        )

        req = self._build_request_async(
            method="PATCH",
            path="/standardjob/adgroup/bulk",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                True,
                "json",
                Optional[models.AdGroupBulkUpdateWorkflowInputWithValidation],
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateAdGroupsJob",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "403", "4XX", "500", "503", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "202", "application/json"):
            return models.UpdateAdGroupsJobResponse(
                standard_job_submit_response=unmarshal_json_response(
                    Optional[models.StandardJobSubmitResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, ["400", "403"], "application/json"):
            response_data = unmarshal_json_response(
                models.ProblemDetailsErrorData, http_res
            )
            raise models.ProblemDetailsError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "503", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class CampaignChannelType(str, Enum):
    NONE = "None"
    DISPLAY = "Display"
    VIDEO = "Video"
    AUDIO = "Audio"
    TV = "TV"
    NATIVE_DISPLAY = "NativeDisplay"
    NATIVE_VIDEO = "NativeVideo"
    DOOH = "DOOH"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .campaignpayload import CampaignPayload, CampaignPayloadTypedDict
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
import pydantic
from ttd_workflows.types import BaseModel
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateCampaignResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    campaign_payload: NotRequired[CampaignPayloadTypedDict]
    r"""Created"""


class CreateCampaignResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    campaign_payload: Optional[CampaignPayload] = None
    r"""Created"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import httpx
import pydantic
from ttd_workflows.types import BaseModel
from typing import Optional
from typing_extensions import Annotated, TypedDict


class HTTPMetadataTypedDict(TypedDict):
    response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""
    request: httpx.Request
    r"""Raw HTTP request; suitable for debugging"""


class HTTPMetadata(BaseModel):
    response: Annotated[Optional[httpx.Response], pydantic.Field(exclude=True)] = None
    r"""Raw HTTP response; suitable for custom response parsing"""

    request: Annotated[Optional[httpx.Request], pydantic.Field(exclude=True)] = None
    r"""Raw HTTP request; suitable for debugging"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class BulkJobStatus(str, Enum):
    QUEUED = "Queued"
    IN_PROGRESS = "InProgress"
    PARTIAL_SUCCESS = "PartialSuccess"
    FAILURE = "Failure"
    SUCCESS = "Success"
    CANCELLED = "Cancelled"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class CustomCPAType(str, Enum):
    DISABLED = "Disabled"
    PIXEL_WEIGHTING = "PixelWeighting"
    CLICK_VIEWTHROUGH_WEIGHTING = "ClickViewthroughWeighting"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class TargetingGender(str, Enum):
    MALE = "Male"
    FEMALE = "Female"
    BOTH = "Both"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from ttd_workflows.types import BaseModel
from typing_extensions import TypedDict


class StandardJobSubmitResponseTypedDict(TypedDict):
    id: int
    r"""The ID of the job."""


class StandardJobSubmitResponse(BaseModel):
    id: int
    r"""The ID of the job."""
